from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v23.common.types.additional_application_info import (
    AdditionalApplicationInfo,
)
from google.ads.googleads.v23.common.types.criteria import (
    DeviceInfo,
    GenderInfo,
    UserInterestInfo,
    UserListInfo,
)
from google.ads.googleads.v23.common.types.dates import DateRange
from google.ads.googleads.v23.enums.types.frequency_cap_time_unit import (
    FrequencyCapTimeUnitEnum,
)
from google.ads.googleads.v23.enums.types.reach_plan_age_range import (
    ReachPlanAgeRangeEnum,
)
from google.ads.googleads.v23.enums.types.reach_plan_conversion_rate_model import (
    ReachPlanConversionRateModelEnum,
)
from google.ads.googleads.v23.enums.types.reach_plan_network import ReachPlanNetworkEnum
from google.ads.googleads.v23.enums.types.reach_plan_plannable_user_list_status import (
    ReachPlanPlannableUserListStatusEnum,
)
from google.ads.googleads.v23.enums.types.reach_plan_surface import ReachPlanSurfaceEnum
from google.ads.googleads.v23.enums.types.target_frequency_time_unit import (
    TargetFrequencyTimeUnitEnum,
)
from google.ads.googleads.v23.enums.types.user_interest_taxonomy_type import (
    UserInterestTaxonomyTypeEnum,
)
from google.ads.googleads.v23.enums.types.user_list_crm_data_source_type import (
    UserListCrmDataSourceTypeEnum,
)
from google.ads.googleads.v23.enums.types.user_list_type import UserListTypeEnum

_M = TypeVar("_M")

class AdvancedProductTargeting(proto.Message):
    surface_targeting_settings: SurfaceTargeting
    target_frequency_settings: TargetFrequencySettings
    youtube_select_settings: YouTubeSelectSettings
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        surface_targeting_settings: SurfaceTargeting = ...,
        target_frequency_settings: TargetFrequencySettings = ...,
        youtube_select_settings: YouTubeSelectSettings = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "surface_targeting_settings",
            "target_frequency_settings",
            "youtube_select_settings",
        ],
    ) -> bool: ...

class AudienceTargeting(proto.Message):
    user_interest: MutableSequence[UserInterestInfo]
    user_lists: MutableSequence[UserListInfo]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        user_interest: MutableSequence[UserInterestInfo] = ...,
        user_lists: MutableSequence[UserListInfo] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["user_interest", "user_lists"]
    ) -> bool: ...

class CampaignDuration(proto.Message):
    duration_in_days: int
    date_range: DateRange
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        duration_in_days: int = ...,
        date_range: DateRange = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["duration_in_days", "date_range"]
    ) -> bool: ...

class ConversionRateSuggestion(proto.Message):
    conversion_rate_model: ReachPlanConversionRateModelEnum.ReachPlanConversionRateModel
    plannable_product_code: str
    surface_targeting: SurfaceTargeting
    conversion_rate: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        conversion_rate_model: ReachPlanConversionRateModelEnum.ReachPlanConversionRateModel = ...,
        plannable_product_code: str = ...,
        surface_targeting: SurfaceTargeting = ...,
        conversion_rate: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "conversion_rate_model",
            "plannable_product_code",
            "surface_targeting",
            "conversion_rate",
        ],
    ) -> bool: ...

class EffectiveFrequencyBreakdown(proto.Message):
    effective_frequency: int
    on_target_reach: int
    total_reach: int
    effective_coview_reach: int
    on_target_effective_coview_reach: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        effective_frequency: int = ...,
        on_target_reach: int = ...,
        total_reach: int = ...,
        effective_coview_reach: int = ...,
        on_target_effective_coview_reach: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "effective_frequency",
            "on_target_reach",
            "total_reach",
            "effective_coview_reach",
            "on_target_effective_coview_reach",
        ],
    ) -> bool: ...

class EffectiveFrequencyLimit(proto.Message):
    effective_frequency_breakdown_limit: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        effective_frequency_breakdown_limit: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["effective_frequency_breakdown_limit"]
    ) -> bool: ...

class Forecast(proto.Message):
    on_target_reach: int
    total_reach: int
    on_target_impressions: int
    total_impressions: int
    viewable_impressions: int
    effective_frequency_breakdowns: MutableSequence[EffectiveFrequencyBreakdown]
    on_target_coview_reach: int
    total_coview_reach: int
    on_target_coview_impressions: int
    total_coview_impressions: int
    conversions: float
    trueview_views: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        on_target_reach: int = ...,
        total_reach: int = ...,
        on_target_impressions: int = ...,
        total_impressions: int = ...,
        viewable_impressions: int = ...,
        effective_frequency_breakdowns: MutableSequence[
            EffectiveFrequencyBreakdown
        ] = ...,
        on_target_coview_reach: int = ...,
        total_coview_reach: int = ...,
        on_target_coview_impressions: int = ...,
        total_coview_impressions: int = ...,
        conversions: float = ...,
        trueview_views: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "on_target_reach",
            "total_reach",
            "on_target_impressions",
            "total_impressions",
            "viewable_impressions",
            "effective_frequency_breakdowns",
            "on_target_coview_reach",
            "total_coview_reach",
            "on_target_coview_impressions",
            "total_coview_impressions",
            "conversions",
            "trueview_views",
        ],
    ) -> bool: ...

class ForecastMetricOptions(proto.Message):
    include_coview: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        include_coview: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["include_coview"]
    ) -> bool: ...

class FrequencyCap(proto.Message):
    impressions: int
    time_unit: FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        impressions: int = ...,
        time_unit: FrequencyCapTimeUnitEnum.FrequencyCapTimeUnit = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["impressions", "time_unit"]
    ) -> bool: ...

class GenerateConversionRatesRequest(proto.Message):
    customer_id: str
    customer_reach_group: str
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        customer_reach_group: str = ...,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal["customer_id", "customer_reach_group", "reach_application_info"],
    ) -> bool: ...

class GenerateConversionRatesResponse(proto.Message):
    conversion_rate_suggestions: MutableSequence[ConversionRateSuggestion]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        conversion_rate_suggestions: MutableSequence[ConversionRateSuggestion] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["conversion_rate_suggestions"]
    ) -> bool: ...

class GenerateReachForecastRequest(proto.Message):
    customer_id: str
    currency_code: str
    campaign_duration: CampaignDuration
    cookie_frequency_cap: int
    cookie_frequency_cap_setting: FrequencyCap
    min_effective_frequency: int
    effective_frequency_limit: EffectiveFrequencyLimit
    targeting: Targeting
    planned_products: MutableSequence[PlannedProduct]
    forecast_metric_options: ForecastMetricOptions
    customer_reach_group: str
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        currency_code: str = ...,
        campaign_duration: CampaignDuration = ...,
        cookie_frequency_cap: int = ...,
        cookie_frequency_cap_setting: FrequencyCap = ...,
        min_effective_frequency: int = ...,
        effective_frequency_limit: EffectiveFrequencyLimit = ...,
        targeting: Targeting = ...,
        planned_products: MutableSequence[PlannedProduct] = ...,
        forecast_metric_options: ForecastMetricOptions = ...,
        customer_reach_group: str = ...,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "currency_code",
            "campaign_duration",
            "cookie_frequency_cap",
            "cookie_frequency_cap_setting",
            "min_effective_frequency",
            "effective_frequency_limit",
            "targeting",
            "planned_products",
            "forecast_metric_options",
            "customer_reach_group",
            "reach_application_info",
        ],
    ) -> bool: ...

class GenerateReachForecastResponse(proto.Message):
    on_target_audience_metrics: OnTargetAudienceMetrics
    reach_curve: ReachCurve
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        on_target_audience_metrics: OnTargetAudienceMetrics = ...,
        reach_curve: ReachCurve = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["on_target_audience_metrics", "reach_curve"]
    ) -> bool: ...

class ListPlannableLocationsRequest(proto.Message):
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["reach_application_info"]
    ) -> bool: ...

class ListPlannableLocationsResponse(proto.Message):
    plannable_locations: MutableSequence[PlannableLocation]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_locations: MutableSequence[PlannableLocation] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["plannable_locations"]
    ) -> bool: ...

class ListPlannableProductsRequest(proto.Message):
    plannable_location_id: str
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_location_id: str = ...,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["plannable_location_id", "reach_application_info"]
    ) -> bool: ...

class ListPlannableProductsResponse(proto.Message):
    product_metadata: MutableSequence[ProductMetadata]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        product_metadata: MutableSequence[ProductMetadata] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["product_metadata"]
    ) -> bool: ...

class ListPlannableUserInterestsRequest(proto.Message):
    customer_id: str
    user_interest_taxonomy_types: MutableSequence[
        UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    ]
    name_query: str
    path_query: str
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        user_interest_taxonomy_types: MutableSequence[
            UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
        ] = ...,
        name_query: str = ...,
        path_query: str = ...,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "user_interest_taxonomy_types",
            "name_query",
            "path_query",
            "reach_application_info",
        ],
    ) -> bool: ...

class ListPlannableUserInterestsResponse(proto.Message):
    plannable_user_interests: MutableSequence[PlannableUserInterest]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_user_interests: MutableSequence[PlannableUserInterest] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["plannable_user_interests"]
    ) -> bool: ...

class ListPlannableUserListsRequest(proto.Message):
    customer_id: str
    customer_reach_group: str
    reach_application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        customer_reach_group: str = ...,
        reach_application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal["customer_id", "customer_reach_group", "reach_application_info"],
    ) -> bool: ...

class ListPlannableUserListsResponse(proto.Message):
    plannable_user_lists: MutableSequence[PlannableUserList]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_user_lists: MutableSequence[PlannableUserList] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["plannable_user_lists"]
    ) -> bool: ...

class OnTargetAudienceMetrics(proto.Message):
    youtube_audience_size: int
    census_audience_size: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        youtube_audience_size: int = ...,
        census_audience_size: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["youtube_audience_size", "census_audience_size"]
    ) -> bool: ...

class PlannableLocation(proto.Message):
    id: str
    name: str
    parent_country_id: int
    country_code: str
    location_type: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        id: str = ...,
        name: str = ...,
        parent_country_id: int = ...,
        country_code: str = ...,
        location_type: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "id", "name", "parent_country_id", "country_code", "location_type"
        ],
    ) -> bool: ...

class PlannableTargeting(proto.Message):
    age_ranges: MutableSequence[ReachPlanAgeRangeEnum.ReachPlanAgeRange]
    genders: MutableSequence[GenderInfo]
    devices: MutableSequence[DeviceInfo]
    networks: MutableSequence[ReachPlanNetworkEnum.ReachPlanNetwork]
    youtube_select_lineups: MutableSequence[YouTubeSelectLineUp]
    surface_targeting: SurfaceTargetingCombinations
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        age_ranges: MutableSequence[ReachPlanAgeRangeEnum.ReachPlanAgeRange] = ...,
        genders: MutableSequence[GenderInfo] = ...,
        devices: MutableSequence[DeviceInfo] = ...,
        networks: MutableSequence[ReachPlanNetworkEnum.ReachPlanNetwork] = ...,
        youtube_select_lineups: MutableSequence[YouTubeSelectLineUp] = ...,
        surface_targeting: SurfaceTargetingCombinations = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "age_ranges",
            "genders",
            "devices",
            "networks",
            "youtube_select_lineups",
            "surface_targeting",
        ],
    ) -> bool: ...

class PlannableUserInterest(proto.Message):
    user_interest: UserInterestInfo
    user_interest_type: UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    user_interest_display_name: str
    user_interest_path: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        user_interest: UserInterestInfo = ...,
        user_interest_type: UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType = ...,
        user_interest_display_name: str = ...,
        user_interest_path: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "user_interest",
            "user_interest_type",
            "user_interest_display_name",
            "user_interest_path",
        ],
    ) -> bool: ...

class PlannableUserList(proto.Message):
    user_list_info: UserListInfo
    display_name: str
    user_list_type: UserListTypeEnum.UserListType
    plannable_status: (
        ReachPlanPlannableUserListStatusEnum.ReachPlanPlannableUserListStatus
    )
    plannable_user_list_metadata: PlannableUserListMetadata
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        user_list_info: UserListInfo = ...,
        display_name: str = ...,
        user_list_type: UserListTypeEnum.UserListType = ...,
        plannable_status: ReachPlanPlannableUserListStatusEnum.ReachPlanPlannableUserListStatus = ...,
        plannable_user_list_metadata: PlannableUserListMetadata = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "user_list_info",
            "display_name",
            "user_list_type",
            "plannable_status",
            "plannable_user_list_metadata",
        ],
    ) -> bool: ...

class PlannableUserListMetadata(proto.Message):
    user_list_crm_data_source_type: (
        UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType
    )
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        user_list_crm_data_source_type: UserListCrmDataSourceTypeEnum.UserListCrmDataSourceType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["user_list_crm_data_source_type"]
    ) -> bool: ...

class PlannedProduct(proto.Message):
    plannable_product_code: str
    budget_micros: int
    conversion_rate: float
    advanced_product_targeting: AdvancedProductTargeting
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_product_code: str = ...,
        budget_micros: int = ...,
        conversion_rate: float = ...,
        advanced_product_targeting: AdvancedProductTargeting = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "plannable_product_code",
            "budget_micros",
            "conversion_rate",
            "advanced_product_targeting",
        ],
    ) -> bool: ...

class PlannedProductForecast(proto.Message):
    on_target_reach: int
    total_reach: int
    on_target_impressions: int
    total_impressions: int
    viewable_impressions: int
    on_target_coview_reach: int
    total_coview_reach: int
    on_target_coview_impressions: int
    total_coview_impressions: int
    average_frequency: float
    conversions: float
    trueview_views: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        on_target_reach: int = ...,
        total_reach: int = ...,
        on_target_impressions: int = ...,
        total_impressions: int = ...,
        viewable_impressions: int = ...,
        on_target_coview_reach: int = ...,
        total_coview_reach: int = ...,
        on_target_coview_impressions: int = ...,
        total_coview_impressions: int = ...,
        average_frequency: float = ...,
        conversions: float = ...,
        trueview_views: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "on_target_reach",
            "total_reach",
            "on_target_impressions",
            "total_impressions",
            "viewable_impressions",
            "on_target_coview_reach",
            "total_coview_reach",
            "on_target_coview_impressions",
            "total_coview_impressions",
            "average_frequency",
            "conversions",
            "trueview_views",
        ],
    ) -> bool: ...

class PlannedProductReachForecast(proto.Message):
    plannable_product_code: str
    cost_micros: int
    planned_product_forecast: PlannedProductForecast
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_product_code: str = ...,
        cost_micros: int = ...,
        planned_product_forecast: PlannedProductForecast = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "plannable_product_code", "cost_micros", "planned_product_forecast"
        ],
    ) -> bool: ...

class ProductMetadata(proto.Message):
    plannable_product_code: str
    plannable_product_name: str
    plannable_targeting: PlannableTargeting
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_product_code: str = ...,
        plannable_product_name: str = ...,
        plannable_targeting: PlannableTargeting = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "plannable_product_code", "plannable_product_name", "plannable_targeting"
        ],
    ) -> bool: ...

class ReachCurve(proto.Message):
    reach_forecasts: MutableSequence[ReachForecast]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        reach_forecasts: MutableSequence[ReachForecast] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["reach_forecasts"]
    ) -> bool: ...

class ReachForecast(proto.Message):
    cost_micros: int
    forecast: Forecast
    planned_product_reach_forecasts: MutableSequence[PlannedProductReachForecast]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        cost_micros: int = ...,
        forecast: Forecast = ...,
        planned_product_reach_forecasts: MutableSequence[
            PlannedProductReachForecast
        ] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["cost_micros", "forecast", "planned_product_reach_forecasts"]
    ) -> bool: ...

class SurfaceTargeting(proto.Message):
    surfaces: MutableSequence[ReachPlanSurfaceEnum.ReachPlanSurface]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        surfaces: MutableSequence[ReachPlanSurfaceEnum.ReachPlanSurface] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["surfaces"]
    ) -> bool: ...

class SurfaceTargetingCombinations(proto.Message):
    default_targeting: SurfaceTargeting
    available_targeting_combinations: MutableSequence[SurfaceTargeting]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        default_targeting: SurfaceTargeting = ...,
        available_targeting_combinations: MutableSequence[SurfaceTargeting] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["default_targeting", "available_targeting_combinations"]
    ) -> bool: ...

class TargetFrequencySettings(proto.Message):
    time_unit: TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit
    target_frequency: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        time_unit: TargetFrequencyTimeUnitEnum.TargetFrequencyTimeUnit = ...,
        target_frequency: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["time_unit", "target_frequency"]
    ) -> bool: ...

class Targeting(proto.Message):
    plannable_location_id: str
    plannable_location_ids: MutableSequence[str]
    age_range: ReachPlanAgeRangeEnum.ReachPlanAgeRange
    genders: MutableSequence[GenderInfo]
    devices: MutableSequence[DeviceInfo]
    network: ReachPlanNetworkEnum.ReachPlanNetwork
    audience_targeting: AudienceTargeting
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        plannable_location_id: str = ...,
        plannable_location_ids: MutableSequence[str] = ...,
        age_range: ReachPlanAgeRangeEnum.ReachPlanAgeRange = ...,
        genders: MutableSequence[GenderInfo] = ...,
        devices: MutableSequence[DeviceInfo] = ...,
        network: ReachPlanNetworkEnum.ReachPlanNetwork = ...,
        audience_targeting: AudienceTargeting = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "plannable_location_id",
            "plannable_location_ids",
            "age_range",
            "genders",
            "devices",
            "network",
            "audience_targeting",
        ],
    ) -> bool: ...

class YouTubeSelectLineUp(proto.Message):
    lineup_id: int
    lineup_name: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        lineup_id: int = ...,
        lineup_name: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["lineup_id", "lineup_name"]
    ) -> bool: ...

class YouTubeSelectSettings(proto.Message):
    lineup_id: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        lineup_id: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["lineup_id"]
    ) -> bool: ...



from .base import (
    AutomaticallyCreatedAssetRemovalServiceTransport as AutomaticallyCreatedAssetRemovalServiceTransport,
)
from .grpc import (
    AutomaticallyCreatedAssetRemovalServiceGrpcTransport as AutomaticallyCreatedAssetRemovalServiceGrpcTransport,
)
from .grpc_asyncio import (
    AutomaticallyCreatedAssetRemovalServiceGrpcAsyncIOTransport as AutomaticallyCreatedAssetRemovalServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AutomaticallyCreatedAssetRemovalServiceTransport",
    "AutomaticallyCreatedAssetRemovalServiceGrpcTransport",
    "AutomaticallyCreatedAssetRemovalServiceGrpcAsyncIOTransport",
]



from .base import (
    AdGroupCriterionLabelServiceTransport as AdGroupCriterionLabelServiceTransport,
)
from .grpc import (
    AdGroupCriterionLabelServiceGrpcTransport as AdGroupCriterionLabelServiceGrpcTransport,
)
from .grpc_asyncio import (
    AdGroupCriterionLabelServiceGrpcAsyncIOTransport as AdGroupCriterionLabelServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AdGroupCriterionLabelServiceTransport",
    "AdGroupCriterionLabelServiceGrpcTransport",
    "AdGroupCriterionLabelServiceGrpcAsyncIOTransport",
]



from .base import ShareablePreviewServiceTransport as ShareablePreviewServiceTransport
from .grpc import (
    ShareablePreviewServiceGrpcTransport as ShareablePreviewServiceGrpcTransport,
)
from .grpc_asyncio import (
    ShareablePreviewServiceGrpcAsyncIOTransport as ShareablePreviewServiceGrpcAsyncIOTransport,
)

__all__ = [
    "ShareablePreviewServiceTransport",
    "ShareablePreviewServiceGrpcTransport",
    "ShareablePreviewServiceGrpcAsyncIOTransport",
]



from .base import DataLinkServiceTransport as DataLinkServiceTransport
from .grpc import DataLinkServiceGrpcTransport as DataLinkServiceGrpcTransport
from .grpc_asyncio import (
    DataLinkServiceGrpcAsyncIOTransport as DataLinkServiceGrpcAsyncIOTransport,
)

__all__ = [
    "DataLinkServiceTransport",
    "DataLinkServiceGrpcTransport",
    "DataLinkServiceGrpcAsyncIOTransport",
]



from collections.abc import Sequence
from typing import Any, overload

import proto
from google.protobuf.message import Message

def get_nested_attr(obj: Any, attr: str, *args: Any) -> Any: ...
def set_nested_message_field(
    message: Message, field_path: str | Sequence[str], value: Any
) -> None: ...
def convert_upper_case_to_snake_case(string: str) -> str: ...
def convert_snake_case_to_upper_case(string: str) -> str: ...
def convert_proto_plus_to_protobuf(message: proto.Message | Message) -> Message: ...
def convert_protobuf_to_proto_plus(
    message: Message | proto.Message,
) -> proto.Message: ...
def proto_copy_from(
    destination: proto.Message | Message, origin: proto.Message | Message
) -> None: ...



from .base import CampaignAssetSetServiceTransport as CampaignAssetSetServiceTransport
from .grpc import (
    CampaignAssetSetServiceGrpcTransport as CampaignAssetSetServiceGrpcTransport,
)
from .grpc_asyncio import (
    CampaignAssetSetServiceGrpcAsyncIOTransport as CampaignAssetSetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CampaignAssetSetServiceTransport",
    "CampaignAssetSetServiceGrpcTransport",
    "CampaignAssetSetServiceGrpcAsyncIOTransport",
]



from .async_client import (
    ShareablePreviewServiceAsyncClient as ShareablePreviewServiceAsyncClient,
)
from .client import ShareablePreviewServiceClient as ShareablePreviewServiceClient

__all__ = ["ShareablePreviewServiceClient", "ShareablePreviewServiceAsyncClient"]



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v23.enums.types.application_instance import (
    ApplicationInstanceEnum,
)

_M = TypeVar("_M")

class AdditionalApplicationInfo(proto.Message):
    application_id: str
    application_instance: ApplicationInstanceEnum.ApplicationInstance
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        application_id: str = ...,
        application_instance: ApplicationInstanceEnum.ApplicationInstance = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["application_id", "application_instance"]
    ) -> bool: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v22.enums.types.recommendation_subscription_status import (
    RecommendationSubscriptionStatusEnum,
)
from google.ads.googleads.v22.enums.types.recommendation_type import (
    RecommendationTypeEnum,
)

_M = TypeVar("_M")

class RecommendationSubscription(proto.Message):
    resource_name: str
    type_: RecommendationTypeEnum.RecommendationType
    create_date_time: str
    modify_date_time: str
    status: RecommendationSubscriptionStatusEnum.RecommendationSubscriptionStatus
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        type_: RecommendationTypeEnum.RecommendationType = ...,
        create_date_time: str = ...,
        modify_date_time: str = ...,
        status: RecommendationSubscriptionStatusEnum.RecommendationSubscriptionStatus = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name", "type_", "create_date_time", "modify_date_time", "status"
        ],
    ) -> bool: ...



from .base import CustomerAssetServiceTransport as CustomerAssetServiceTransport
from .grpc import CustomerAssetServiceGrpcTransport as CustomerAssetServiceGrpcTransport
from .grpc_asyncio import (
    CustomerAssetServiceGrpcAsyncIOTransport as CustomerAssetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerAssetServiceTransport",
    "CustomerAssetServiceGrpcTransport",
    "CustomerAssetServiceGrpcAsyncIOTransport",
]



from .async_client import (
    AssetGroupSignalServiceAsyncClient as AssetGroupSignalServiceAsyncClient,
)
from .client import AssetGroupSignalServiceClient as AssetGroupSignalServiceClient

__all__ = ["AssetGroupSignalServiceClient", "AssetGroupSignalServiceAsyncClient"]



from .async_client import (
    KeywordPlanAdGroupServiceAsyncClient as KeywordPlanAdGroupServiceAsyncClient,
)
from .client import KeywordPlanAdGroupServiceClient as KeywordPlanAdGroupServiceClient

__all__ = ["KeywordPlanAdGroupServiceClient", "KeywordPlanAdGroupServiceAsyncClient"]



from .base import (
    ConversionCustomVariableServiceTransport as ConversionCustomVariableServiceTransport,
)
from .grpc import (
    ConversionCustomVariableServiceGrpcTransport as ConversionCustomVariableServiceGrpcTransport,
)
from .grpc_asyncio import (
    ConversionCustomVariableServiceGrpcAsyncIOTransport as ConversionCustomVariableServiceGrpcAsyncIOTransport,
)

__all__ = [
    "ConversionCustomVariableServiceTransport",
    "ConversionCustomVariableServiceGrpcTransport",
    "ConversionCustomVariableServiceGrpcAsyncIOTransport",
]



from .async_client import SharedSetServiceAsyncClient as SharedSetServiceAsyncClient
from .client import SharedSetServiceClient as SharedSetServiceClient

__all__ = ["SharedSetServiceClient", "SharedSetServiceAsyncClient"]



from .async_client import (
    KeywordPlanAdGroupKeywordServiceAsyncClient as KeywordPlanAdGroupKeywordServiceAsyncClient,
)
from .client import (
    KeywordPlanAdGroupKeywordServiceClient as KeywordPlanAdGroupKeywordServiceClient,
)

__all__ = [
    "KeywordPlanAdGroupKeywordServiceClient",
    "KeywordPlanAdGroupKeywordServiceAsyncClient",
]



from .base import CampaignAssetServiceTransport as CampaignAssetServiceTransport
from .grpc import CampaignAssetServiceGrpcTransport as CampaignAssetServiceGrpcTransport
from .grpc_asyncio import (
    CampaignAssetServiceGrpcAsyncIOTransport as CampaignAssetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CampaignAssetServiceTransport",
    "CampaignAssetServiceGrpcTransport",
    "CampaignAssetServiceGrpcAsyncIOTransport",
]



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v21.enums.types.geo_target_constant_status import (
    GeoTargetConstantStatusEnum,
)

_M = TypeVar("_M")

class GeoTargetConstant(proto.Message):
    resource_name: str
    id: int
    name: str
    country_code: str
    target_type: str
    status: GeoTargetConstantStatusEnum.GeoTargetConstantStatus
    canonical_name: str
    parent_geo_target: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        id: int = ...,
        name: str = ...,
        country_code: str = ...,
        target_type: str = ...,
        status: GeoTargetConstantStatusEnum.GeoTargetConstantStatus = ...,
        canonical_name: str = ...,
        parent_geo_target: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "id",
            "name",
            "country_code",
            "target_type",
            "status",
            "canonical_name",
            "parent_geo_target",
        ],
    ) -> bool: ...



from .async_client import (
    CustomerConversionGoalServiceAsyncClient as CustomerConversionGoalServiceAsyncClient,
)
from .client import (
    CustomerConversionGoalServiceClient as CustomerConversionGoalServiceClient,
)

__all__ = [
    "CustomerConversionGoalServiceClient",
    "CustomerConversionGoalServiceAsyncClient",
]



from .base import CustomerServiceTransport as CustomerServiceTransport
from .grpc import CustomerServiceGrpcTransport as CustomerServiceGrpcTransport
from .grpc_asyncio import (
    CustomerServiceGrpcAsyncIOTransport as CustomerServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerServiceTransport",
    "CustomerServiceGrpcTransport",
    "CustomerServiceGrpcAsyncIOTransport",
]



from .base import AdParameterServiceTransport as AdParameterServiceTransport
from .grpc import AdParameterServiceGrpcTransport as AdParameterServiceGrpcTransport
from .grpc_asyncio import (
    AdParameterServiceGrpcAsyncIOTransport as AdParameterServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AdParameterServiceTransport",
    "AdParameterServiceGrpcTransport",
    "AdParameterServiceGrpcAsyncIOTransport",
]



from .base import GoogleAdsServiceTransport as GoogleAdsServiceTransport
from .grpc import GoogleAdsServiceGrpcTransport as GoogleAdsServiceGrpcTransport
from .grpc_asyncio import (
    GoogleAdsServiceGrpcAsyncIOTransport as GoogleAdsServiceGrpcAsyncIOTransport,
)

__all__ = [
    "GoogleAdsServiceTransport",
    "GoogleAdsServiceGrpcTransport",
    "GoogleAdsServiceGrpcAsyncIOTransport",
]



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

_M = TypeVar("_M")

class KeywordPlanAdGroup(proto.Message):
    resource_name: str
    keyword_plan_campaign: str
    id: int
    name: str
    cpc_bid_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        keyword_plan_campaign: str = ...,
        id: int = ...,
        name: str = ...,
        cpc_bid_micros: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name", "keyword_plan_campaign", "id", "name", "cpc_bid_micros"
        ],
    ) -> bool: ...



from .base import (
    CampaignGoalConfigServiceTransport as CampaignGoalConfigServiceTransport,
)
from .grpc import (
    CampaignGoalConfigServiceGrpcTransport as CampaignGoalConfigServiceGrpcTransport,
)
from .grpc_asyncio import (
    CampaignGoalConfigServiceGrpcAsyncIOTransport as CampaignGoalConfigServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CampaignGoalConfigServiceTransport",
    "CampaignGoalConfigServiceGrpcTransport",
    "CampaignGoalConfigServiceGrpcAsyncIOTransport",
]



from .async_client import (
    UserListCustomerTypeServiceAsyncClient as UserListCustomerTypeServiceAsyncClient,
)
from .client import (
    UserListCustomerTypeServiceClient as UserListCustomerTypeServiceClient,
)

__all__ = [
    "UserListCustomerTypeServiceClient",
    "UserListCustomerTypeServiceAsyncClient",
]



from .async_client import (
    BrandSuggestionServiceAsyncClient as BrandSuggestionServiceAsyncClient,
)
from .client import BrandSuggestionServiceClient as BrandSuggestionServiceClient

__all__ = ["BrandSuggestionServiceClient", "BrandSuggestionServiceAsyncClient"]



from .async_client import (
    AdGroupCriterionServiceAsyncClient as AdGroupCriterionServiceAsyncClient,
)
from .client import AdGroupCriterionServiceClient as AdGroupCriterionServiceClient

__all__ = ["AdGroupCriterionServiceClient", "AdGroupCriterionServiceAsyncClient"]



from .async_client import (
    RecommendationServiceAsyncClient as RecommendationServiceAsyncClient,
)
from .client import RecommendationServiceClient as RecommendationServiceClient

__all__ = ["RecommendationServiceClient", "RecommendationServiceAsyncClient"]



from .async_client import (
    CampaignLabelServiceAsyncClient as CampaignLabelServiceAsyncClient,
)
from .client import CampaignLabelServiceClient as CampaignLabelServiceClient

__all__ = ["CampaignLabelServiceClient", "CampaignLabelServiceAsyncClient"]



from .async_client import (
    BiddingDataExclusionServiceAsyncClient as BiddingDataExclusionServiceAsyncClient,
)
from .client import (
    BiddingDataExclusionServiceClient as BiddingDataExclusionServiceClient,
)

__all__ = [
    "BiddingDataExclusionServiceClient",
    "BiddingDataExclusionServiceAsyncClient",
]



from .async_client import (
    OfflineUserDataJobServiceAsyncClient as OfflineUserDataJobServiceAsyncClient,
)
from .client import OfflineUserDataJobServiceClient as OfflineUserDataJobServiceClient

__all__ = ["OfflineUserDataJobServiceClient", "OfflineUserDataJobServiceAsyncClient"]



from .base import ConversionActionServiceTransport as ConversionActionServiceTransport
from .grpc import (
    ConversionActionServiceGrpcTransport as ConversionActionServiceGrpcTransport,
)
from .grpc_asyncio import (
    ConversionActionServiceGrpcAsyncIOTransport as ConversionActionServiceGrpcAsyncIOTransport,
)

__all__ = [
    "ConversionActionServiceTransport",
    "ConversionActionServiceGrpcTransport",
    "ConversionActionServiceGrpcAsyncIOTransport",
]



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v23.common.types.additional_application_info import (
    AdditionalApplicationInfo,
)
from google.ads.googleads.v23.common.types.audience_insights_attribute import (
    AudienceInsightsAttribute,
    AudienceInsightsAttributeMetadata,
    AudienceInsightsEntity,
)
from google.ads.googleads.v23.common.types.criteria import (
    LocationInfo,
    YouTubeChannelInfo,
)
from google.ads.googleads.v23.enums.types.insights_trend import InsightsTrendEnum

_M = TypeVar("_M")

class GenerateCreatorInsightsRequest(proto.Message):
    class SearchAttributes(proto.Message):
        audience_attributes: MutableSequence[AudienceInsightsAttribute]
        creator_attributes: MutableSequence[AudienceInsightsAttribute]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            audience_attributes: MutableSequence[AudienceInsightsAttribute] = ...,
            creator_attributes: MutableSequence[AudienceInsightsAttribute] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["audience_attributes", "creator_attributes"]
        ) -> bool: ...

    class SearchBrand(proto.Message):
        brand_entities: MutableSequence[AudienceInsightsAttribute]
        include_related_topics: bool
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            brand_entities: MutableSequence[AudienceInsightsAttribute] = ...,
            include_related_topics: bool = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["brand_entities", "include_related_topics"]
        ) -> bool: ...

    class YouTubeChannels(proto.Message):
        youtube_channels: MutableSequence[YouTubeChannelInfo]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            youtube_channels: MutableSequence[YouTubeChannelInfo] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["youtube_channels"]
        ) -> bool: ...

    customer_id: str
    customer_insights_group: str
    insights_application_info: AdditionalApplicationInfo
    country_locations: MutableSequence[LocationInfo]
    sub_country_locations: MutableSequence[LocationInfo]
    search_attributes: GenerateCreatorInsightsRequest.SearchAttributes
    search_brand: GenerateCreatorInsightsRequest.SearchBrand
    search_channels: GenerateCreatorInsightsRequest.YouTubeChannels
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        customer_insights_group: str = ...,
        insights_application_info: AdditionalApplicationInfo = ...,
        country_locations: MutableSequence[LocationInfo] = ...,
        sub_country_locations: MutableSequence[LocationInfo] = ...,
        search_attributes: GenerateCreatorInsightsRequest.SearchAttributes = ...,
        search_brand: GenerateCreatorInsightsRequest.SearchBrand = ...,
        search_channels: GenerateCreatorInsightsRequest.YouTubeChannels = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "customer_insights_group",
            "insights_application_info",
            "country_locations",
            "sub_country_locations",
            "search_attributes",
            "search_brand",
            "search_channels",
        ],
    ) -> bool: ...

class GenerateCreatorInsightsResponse(proto.Message):
    creator_insights: MutableSequence[YouTubeCreatorInsights]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        creator_insights: MutableSequence[YouTubeCreatorInsights] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["creator_insights"]
    ) -> bool: ...

class GenerateTrendingInsightsRequest(proto.Message):
    customer_id: str
    customer_insights_group: str
    insights_application_info: AdditionalApplicationInfo
    country_location: LocationInfo
    search_audience: SearchAudience
    search_topics: SearchTopics
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        customer_insights_group: str = ...,
        insights_application_info: AdditionalApplicationInfo = ...,
        country_location: LocationInfo = ...,
        search_audience: SearchAudience = ...,
        search_topics: SearchTopics = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "customer_insights_group",
            "insights_application_info",
            "country_location",
            "search_audience",
            "search_topics",
        ],
    ) -> bool: ...

class GenerateTrendingInsightsResponse(proto.Message):
    trend_insights: MutableSequence[TrendInsight]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        trend_insights: MutableSequence[TrendInsight] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["trend_insights"]
    ) -> bool: ...

class LanguageDistribution(proto.Message):
    language_code: str
    proportion: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        language_code: str = ...,
        proportion: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["language_code", "proportion"]
    ) -> bool: ...

class SearchAudience(proto.Message):
    audience_attributes: MutableSequence[AudienceInsightsAttribute]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        audience_attributes: MutableSequence[AudienceInsightsAttribute] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["audience_attributes"]
    ) -> bool: ...

class SearchTopics(proto.Message):
    entities: MutableSequence[AudienceInsightsEntity]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        entities: MutableSequence[AudienceInsightsEntity] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["entities"]
    ) -> bool: ...

class TrendInsight(proto.Message):
    trend_attribute: AudienceInsightsAttributeMetadata
    trend_metrics: TrendInsightMetrics
    trend: InsightsTrendEnum.InsightsTrend
    related_videos: MutableSequence[AudienceInsightsAttributeMetadata]
    related_creators: MutableSequence[YouTubeCreatorInsights]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        trend_attribute: AudienceInsightsAttributeMetadata = ...,
        trend_metrics: TrendInsightMetrics = ...,
        trend: InsightsTrendEnum.InsightsTrend = ...,
        related_videos: MutableSequence[AudienceInsightsAttributeMetadata] = ...,
        related_creators: MutableSequence[YouTubeCreatorInsights] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "trend_attribute",
            "trend_metrics",
            "trend",
            "related_videos",
            "related_creators",
        ],
    ) -> bool: ...

class TrendInsightMetrics(proto.Message):
    views_count: int
    audience_share: float
    trend_change_percent: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        views_count: int = ...,
        audience_share: float = ...,
        trend_change_percent: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["views_count", "audience_share", "trend_change_percent"]
    ) -> bool: ...

class YouTubeChannelInsights(proto.Message):
    display_name: str
    youtube_channel: YouTubeChannelInfo
    channel_url: str
    channel_description: str
    handle: str
    thumbnail_url: str
    publish_date: str
    country_location: LocationInfo
    channel_metrics: YouTubeMetrics
    channel_audience_attributes: MutableSequence[AudienceInsightsAttributeMetadata]
    channel_attributes: MutableSequence[AudienceInsightsAttributeMetadata]
    top_videos: MutableSequence[AudienceInsightsAttributeMetadata]
    language_distributions: MutableSequence[LanguageDistribution]
    channel_type: str
    relevance_score: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        display_name: str = ...,
        youtube_channel: YouTubeChannelInfo = ...,
        channel_url: str = ...,
        channel_description: str = ...,
        handle: str = ...,
        thumbnail_url: str = ...,
        publish_date: str = ...,
        country_location: LocationInfo = ...,
        channel_metrics: YouTubeMetrics = ...,
        channel_audience_attributes: MutableSequence[
            AudienceInsightsAttributeMetadata
        ] = ...,
        channel_attributes: MutableSequence[AudienceInsightsAttributeMetadata] = ...,
        top_videos: MutableSequence[AudienceInsightsAttributeMetadata] = ...,
        language_distributions: MutableSequence[LanguageDistribution] = ...,
        channel_type: str = ...,
        relevance_score: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "display_name",
            "youtube_channel",
            "channel_url",
            "channel_description",
            "handle",
            "thumbnail_url",
            "publish_date",
            "country_location",
            "channel_metrics",
            "channel_audience_attributes",
            "channel_attributes",
            "top_videos",
            "language_distributions",
            "channel_type",
            "relevance_score",
        ],
    ) -> bool: ...

class YouTubeCreatorInsights(proto.Message):
    creator_name: str
    creator_channels: MutableSequence[YouTubeChannelInsights]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        creator_name: str = ...,
        creator_channels: MutableSequence[YouTubeChannelInsights] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["creator_name", "creator_channels"]
    ) -> bool: ...

class YouTubeMetrics(proto.Message):
    subscriber_count: int
    views_count: int
    video_count: int
    likes_count: int
    shares_count: int
    comments_count: int
    engagement_rate: float
    average_views_per_video: float
    average_likes_per_video: float
    average_shares_per_video: float
    average_comments_per_video: float
    shorts_views_count: int
    shorts_video_count: int
    is_active_shorts_creator: bool
    is_active_live_stream_creator: bool
    is_brand_connect_creator: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        subscriber_count: int = ...,
        views_count: int = ...,
        video_count: int = ...,
        likes_count: int = ...,
        shares_count: int = ...,
        comments_count: int = ...,
        engagement_rate: float = ...,
        average_views_per_video: float = ...,
        average_likes_per_video: float = ...,
        average_shares_per_video: float = ...,
        average_comments_per_video: float = ...,
        shorts_views_count: int = ...,
        shorts_video_count: int = ...,
        is_active_shorts_creator: bool = ...,
        is_active_live_stream_creator: bool = ...,
        is_brand_connect_creator: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "subscriber_count",
            "views_count",
            "video_count",
            "likes_count",
            "shares_count",
            "comments_count",
            "engagement_rate",
            "average_views_per_video",
            "average_likes_per_video",
            "average_shares_per_video",
            "average_comments_per_video",
            "shorts_views_count",
            "shorts_video_count",
            "is_active_shorts_creator",
            "is_active_live_stream_creator",
            "is_brand_connect_creator",
        ],
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v23.common.types.additional_application_info import (
    AdditionalApplicationInfo,
)
from google.ads.googleads.v23.common.types.criteria import LocationInfo
from google.ads.googleads.v23.common.types.dates import DateRange
from google.ads.googleads.v23.enums.types.benchmarks_marketing_objective import (
    BenchmarksMarketingObjectiveEnum,
)
from google.ads.googleads.v23.enums.types.benchmarks_source_type import (
    BenchmarksSourceTypeEnum,
)

_M = TypeVar("_M")

class BenchmarksLocation(proto.Message):
    location_name: str
    location_type: str
    location_info: LocationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        location_name: str = ...,
        location_type: str = ...,
        location_info: LocationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["location_name", "location_type", "location_info"]
    ) -> bool: ...

class BenchmarksProductMetadata(proto.Message):
    product_name: str
    product_code: str
    marketing_objective: BenchmarksMarketingObjectiveEnum.BenchmarksMarketingObjective
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        product_name: str = ...,
        product_code: str = ...,
        marketing_objective: BenchmarksMarketingObjectiveEnum.BenchmarksMarketingObjective = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["product_name", "product_code", "marketing_objective"]
    ) -> bool: ...

class BenchmarksSource(proto.Message):
    industry_vertical_id: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        industry_vertical_id: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["industry_vertical_id"]
    ) -> bool: ...

class BenchmarksSourceMetadata(proto.Message):
    benchmarks_source_type: BenchmarksSourceTypeEnum.BenchmarksSourceType
    industry_vertical_info: IndustryVerticalInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        benchmarks_source_type: BenchmarksSourceTypeEnum.BenchmarksSourceType = ...,
        industry_vertical_info: IndustryVerticalInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["benchmarks_source_type", "industry_vertical_info"]
    ) -> bool: ...

class GenerateBenchmarksMetricsRequest(proto.Message):
    customer_id: str
    date_range: DateRange
    location: LocationInfo
    benchmarks_source: BenchmarksSource
    product_filter: ProductFilter
    currency_code: str
    customer_benchmarks_group: str
    application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        date_range: DateRange = ...,
        location: LocationInfo = ...,
        benchmarks_source: BenchmarksSource = ...,
        product_filter: ProductFilter = ...,
        currency_code: str = ...,
        customer_benchmarks_group: str = ...,
        application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "date_range",
            "location",
            "benchmarks_source",
            "product_filter",
            "currency_code",
            "customer_benchmarks_group",
            "application_info",
        ],
    ) -> bool: ...

class GenerateBenchmarksMetricsResponse(proto.Message):
    customer_metrics: Metrics
    average_benchmarks_metrics: Metrics
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_metrics: Metrics = ...,
        average_benchmarks_metrics: Metrics = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["customer_metrics", "average_benchmarks_metrics"]
    ) -> bool: ...

class IndustryVerticalInfo(proto.Message):
    industry_vertical_name: str
    industry_vertical_id: int
    parent_industry_vertical_id: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        industry_vertical_name: str = ...,
        industry_vertical_id: int = ...,
        parent_industry_vertical_id: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "industry_vertical_name",
            "industry_vertical_id",
            "parent_industry_vertical_id",
        ],
    ) -> bool: ...

class ListBenchmarksAvailableDatesRequest(proto.Message):
    application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["application_info"]
    ) -> bool: ...

class ListBenchmarksAvailableDatesResponse(proto.Message):
    supported_dates: DateRange
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        supported_dates: DateRange = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["supported_dates"]
    ) -> bool: ...

class ListBenchmarksLocationsRequest(proto.Message):
    application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["application_info"]
    ) -> bool: ...

class ListBenchmarksLocationsResponse(proto.Message):
    benchmarks_locations: MutableSequence[BenchmarksLocation]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        benchmarks_locations: MutableSequence[BenchmarksLocation] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["benchmarks_locations"]
    ) -> bool: ...

class ListBenchmarksProductsRequest(proto.Message):
    application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["application_info"]
    ) -> bool: ...

class ListBenchmarksProductsResponse(proto.Message):
    benchmarks_products: MutableSequence[BenchmarksProductMetadata]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        benchmarks_products: MutableSequence[BenchmarksProductMetadata] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["benchmarks_products"]
    ) -> bool: ...

class ListBenchmarksSourcesRequest(proto.Message):
    benchmarks_sources: MutableSequence[BenchmarksSourceTypeEnum.BenchmarksSourceType]
    application_info: AdditionalApplicationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        benchmarks_sources: MutableSequence[
            BenchmarksSourceTypeEnum.BenchmarksSourceType
        ] = ...,
        application_info: AdditionalApplicationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["benchmarks_sources", "application_info"]
    ) -> bool: ...

class ListBenchmarksSourcesResponse(proto.Message):
    benchmarks_sources: MutableSequence[BenchmarksSourceMetadata]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        benchmarks_sources: MutableSequence[BenchmarksSourceMetadata] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["benchmarks_sources"]
    ) -> bool: ...

class Metrics(proto.Message):
    average_rate_metrics: RateMetrics
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        average_rate_metrics: RateMetrics = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["average_rate_metrics"]
    ) -> bool: ...

class ProductFilter(proto.Message):
    class MarketingObjectiveList(proto.Message):
        marketing_objectives: MutableSequence[
            BenchmarksMarketingObjectiveEnum.BenchmarksMarketingObjective
        ]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            marketing_objectives: MutableSequence[
                BenchmarksMarketingObjectiveEnum.BenchmarksMarketingObjective
            ] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["marketing_objectives"]
        ) -> bool: ...

    class ProductList(proto.Message):
        product_codes: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            product_codes: MutableSequence[str] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["product_codes"]
        ) -> bool: ...

    product_list: ProductFilter.ProductList
    marketing_objective_list: ProductFilter.MarketingObjectiveList
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        product_list: ProductFilter.ProductList = ...,
        marketing_objective_list: ProductFilter.MarketingObjectiveList = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["product_list", "marketing_objective_list"]
    ) -> bool: ...

class RateMetrics(proto.Message):
    average_cpm: float
    average_active_view_cpm: float
    trueview_average_cpv: float
    average_cpc: float
    average_cpi: float
    average_cpe: float
    interaction_rate: float
    engagement_rate: float
    active_view_viewability: float
    trueview_view_rate: float
    click_through_rate: float
    video_completion_p25_rate: float
    video_completion_p50_rate: float
    video_completion_p75_rate: float
    video_completion_p100_rate: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        average_cpm: float = ...,
        average_active_view_cpm: float = ...,
        trueview_average_cpv: float = ...,
        average_cpc: float = ...,
        average_cpi: float = ...,
        average_cpe: float = ...,
        interaction_rate: float = ...,
        engagement_rate: float = ...,
        active_view_viewability: float = ...,
        trueview_view_rate: float = ...,
        click_through_rate: float = ...,
        video_completion_p25_rate: float = ...,
        video_completion_p50_rate: float = ...,
        video_completion_p75_rate: float = ...,
        video_completion_p100_rate: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "average_cpm",
            "average_active_view_cpm",
            "trueview_average_cpv",
            "average_cpc",
            "average_cpi",
            "average_cpe",
            "interaction_rate",
            "engagement_rate",
            "active_view_viewability",
            "trueview_view_rate",
            "click_through_rate",
            "video_completion_p25_rate",
            "video_completion_p50_rate",
            "video_completion_p75_rate",
            "video_completion_p100_rate",
        ],
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

_M = TypeVar("_M")

class UrlCollection(proto.Message):
    url_collection_id: str
    final_urls: MutableSequence[str]
    final_mobile_urls: MutableSequence[str]
    tracking_url_template: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        url_collection_id: str = ...,
        final_urls: MutableSequence[str] = ...,
        final_mobile_urls: MutableSequence[str] = ...,
        tracking_url_template: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "url_collection_id",
            "final_urls",
            "final_mobile_urls",
            "tracking_url_template",
        ],
    ) -> bool: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v20.enums.types.keyword_match_type import KeywordMatchTypeEnum

_M = TypeVar("_M")

class KeywordPlanCampaignKeyword(proto.Message):
    resource_name: str
    keyword_plan_campaign: str
    id: int
    text: str
    match_type: KeywordMatchTypeEnum.KeywordMatchType
    negative: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        keyword_plan_campaign: str = ...,
        id: int = ...,
        text: str = ...,
        match_type: KeywordMatchTypeEnum.KeywordMatchType = ...,
        negative: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "keyword_plan_campaign",
            "id",
            "text",
            "match_type",
            "negative",
        ],
    ) -> bool: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v20.enums.types.asset_set_link_status import (
    AssetSetLinkStatusEnum,
)

_M = TypeVar("_M")

class CustomerAssetSet(proto.Message):
    resource_name: str
    asset_set: str
    customer: str
    status: AssetSetLinkStatusEnum.AssetSetLinkStatus
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        asset_set: str = ...,
        customer: str = ...,
        status: AssetSetLinkStatusEnum.AssetSetLinkStatus = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name", "asset_set", "customer", "status"]
    ) -> bool: ...



from typing import Callable, MutableSequence, Sequence

from _typeshed import Incomplete
from google.api_core import gapic_v1, operation_async, retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials

from google.ads.googleads.v22.services.services.experiment_service import pagers
from google.ads.googleads.v22.services.types import experiment_service

from .transports.base import ExperimentServiceTransport

__all__ = ["ExperimentServiceAsyncClient"]

class ExperimentServiceAsyncClient:
    DEFAULT_ENDPOINT: Incomplete
    DEFAULT_MTLS_ENDPOINT: Incomplete
    campaign_path: Incomplete
    parse_campaign_path: Incomplete
    campaign_budget_path: Incomplete
    parse_campaign_budget_path: Incomplete
    experiment_path: Incomplete
    parse_experiment_path: Incomplete
    common_billing_account_path: Incomplete
    parse_common_billing_account_path: Incomplete
    common_folder_path: Incomplete
    parse_common_folder_path: Incomplete
    common_organization_path: Incomplete
    parse_common_organization_path: Incomplete
    common_project_path: Incomplete
    parse_common_project_path: Incomplete
    common_location_path: Incomplete
    parse_common_location_path: Incomplete
    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs): ...
    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs): ...
    from_service_account_json = from_service_account_file
    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: ClientOptions | None = None
    ): ...
    @property
    def transport(self) -> ExperimentServiceTransport: ...
    @property
    def api_endpoint(self): ...
    @property
    def universe_domain(self) -> str: ...
    get_transport_class: Incomplete
    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials | None = None,
        transport: str
        | ExperimentServiceTransport
        | Callable[..., ExperimentServiceTransport]
        | None = "grpc_asyncio",
        client_options: ClientOptions | None = None,
        client_info: gapic_v1.client_info.ClientInfo = ...,
    ) -> None: ...
    async def mutate_experiments(
        self,
        request: experiment_service.MutateExperimentsRequest | dict | None = None,
        *,
        customer_id: str | None = None,
        operations: MutableSequence[experiment_service.ExperimentOperation]
        | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> experiment_service.MutateExperimentsResponse: ...
    async def end_experiment(
        self,
        request: experiment_service.EndExperimentRequest | dict | None = None,
        *,
        experiment: str | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> None: ...
    async def list_experiment_async_errors(
        self,
        request: experiment_service.ListExperimentAsyncErrorsRequest
        | dict
        | None = None,
        *,
        resource_name: str | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> pagers.ListExperimentAsyncErrorsAsyncPager: ...
    async def graduate_experiment(
        self,
        request: experiment_service.GraduateExperimentRequest | dict | None = None,
        *,
        experiment: str | None = None,
        campaign_budget_mappings: MutableSequence[
            experiment_service.CampaignBudgetMapping
        ]
        | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> None: ...
    async def schedule_experiment(
        self,
        request: experiment_service.ScheduleExperimentRequest | dict | None = None,
        *,
        resource_name: str | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> operation_async.AsyncOperation: ...
    async def promote_experiment(
        self,
        request: experiment_service.PromoteExperimentRequest | dict | None = None,
        *,
        resource_name: str | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> operation_async.AsyncOperation: ...
    async def __aenter__(self) -> ExperimentServiceAsyncClient: ...
    async def __aexit__(self, exc_type, exc, tb) -> None: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

_M = TypeVar("_M")

class UserLocationView(proto.Message):
    resource_name: str
    country_criterion_id: int
    targeting_location: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        country_criterion_id: int = ...,
        targeting_location: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal["resource_name", "country_criterion_id", "targeting_location"],
    ) -> bool: ...



from .async_client import ProductLinkServiceAsyncClient as ProductLinkServiceAsyncClient
from .client import ProductLinkServiceClient as ProductLinkServiceClient

__all__ = ["ProductLinkServiceClient", "ProductLinkServiceAsyncClient"]



from collections.abc import Callable
from typing import TypeVar

import grpc

# Autogenerated imports
import google.ads.googleads.v20.services
import google.ads.googleads.v21.services
import google.ads.googleads.v22.services
import google.ads.googleads.v23.services

# End of autogenerated imports
from .interceptor import Interceptor

_Request = TypeVar(
    "_Request",
    # Autogenerated request types
    google.ads.googleads.v20.services.SearchGoogleAdsRequest,
    google.ads.googleads.v20.services.SearchGoogleAdsStreamRequest,
    google.ads.googleads.v21.services.SearchGoogleAdsRequest,
    google.ads.googleads.v21.services.SearchGoogleAdsStreamRequest,
    google.ads.googleads.v22.services.SearchGoogleAdsRequest,
    google.ads.googleads.v22.services.SearchGoogleAdsStreamRequest,
    google.ads.googleads.v23.services.SearchGoogleAdsRequest,
    google.ads.googleads.v23.services.SearchGoogleAdsStreamRequest,
    # End of autogenerated request types
)
_Response = TypeVar("_Response")

class ExceptionInterceptor(
    Interceptor, grpc.UnaryUnaryClientInterceptor, grpc.UnaryStreamClientInterceptor
):
    def __init__(self, api_version: str, use_proto_plus: bool = False) -> None: ...
    def intercept_unary_unary(
        self,
        continuation: Callable[[grpc.ClientCallDetails, _Request], _Response],
        client_call_details: grpc.ClientCallDetails,
        request: _Request,
    ) -> _Response: ...
    def intercept_unary_stream(
        self,
        continuation: Callable[[grpc.ClientCallDetails, _Request], _Response],
        client_call_details: grpc.ClientCallDetails,
        request: _Request,
    ) -> _Response: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v21.enums.types.customizer_attribute_status import (
    CustomizerAttributeStatusEnum,
)
from google.ads.googleads.v21.enums.types.customizer_attribute_type import (
    CustomizerAttributeTypeEnum,
)

_M = TypeVar("_M")

class CustomizerAttribute(proto.Message):
    resource_name: str
    id: int
    name: str
    type_: CustomizerAttributeTypeEnum.CustomizerAttributeType
    status: CustomizerAttributeStatusEnum.CustomizerAttributeStatus
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        id: int = ...,
        name: str = ...,
        type_: CustomizerAttributeTypeEnum.CustomizerAttributeType = ...,
        status: CustomizerAttributeStatusEnum.CustomizerAttributeStatus = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name", "id", "name", "type_", "status"]
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v23.enums.types.interaction_event_type import (
    InteractionEventTypeEnum,
)
from google.ads.googleads.v23.enums.types.quality_score_bucket import (
    QualityScoreBucketEnum,
)

_M = TypeVar("_M")

class Metrics(proto.Message):
    absolute_top_impression_percentage: float
    active_view_cpm: float
    active_view_ctr: float
    active_view_impressions: int
    active_view_measurability: float
    active_view_measurable_cost_micros: int
    active_view_measurable_impressions: int
    active_view_viewability: float
    all_conversions_from_interactions_rate: float
    all_conversions_value: float
    all_conversions_value_by_conversion_date: float
    all_new_customer_lifetime_value: float
    all_conversions: float
    all_conversions_by_conversion_date: float
    all_conversions_value_per_cost: float
    all_conversions_from_click_to_call: float
    all_conversions_from_directions: float
    all_conversions_from_interactions_value_per_interaction: float
    all_conversions_from_menu: float
    all_conversions_from_order: float
    all_conversions_from_other_engagement: float
    all_conversions_from_store_visit: float
    all_conversions_from_store_website: float
    auction_insight_search_absolute_top_impression_percentage: float
    auction_insight_search_impression_share: float
    auction_insight_search_outranking_share: float
    auction_insight_search_overlap_rate: float
    auction_insight_search_position_above_rate: float
    auction_insight_search_top_impression_percentage: float
    average_cost: float
    average_cpc: float
    average_cpe: float
    average_cpm: float
    trueview_average_cpv: float
    average_page_views: float
    average_time_on_site: float
    benchmark_average_max_cpc: float
    biddable_app_install_conversions: float
    biddable_app_post_install_conversions: float
    biddable_cohort_app_post_install_conversions: float
    benchmark_ctr: float
    bounce_rate: float
    clicks: int
    combined_clicks: int
    combined_clicks_per_query: float
    combined_queries: int
    content_budget_lost_impression_share: float
    content_impression_share: float
    conversion_last_received_request_date_time: str
    conversion_last_conversion_date: str
    content_rank_lost_impression_share: float
    conversions_from_interactions_rate: float
    conversions_value: float
    conversions_value_by_conversion_date: float
    new_customer_lifetime_value: float
    conversions_value_per_cost: float
    conversions_from_interactions_value_per_interaction: float
    conversions: float
    conversions_by_conversion_date: float
    cost_micros: int
    cost_per_all_conversions: float
    cost_per_conversion: float
    cost_per_current_model_attributed_conversion: float
    cross_device_conversions: float
    cross_device_conversions_by_conversion_date: float
    cross_device_conversions_value: float
    cross_device_conversions_value_micros: int
    cross_device_conversions_value_by_conversion_date: float
    ctr: float
    current_model_attributed_conversions: float
    current_model_attributed_conversions_from_interactions_rate: float
    current_model_attributed_conversions_from_interactions_value_per_interaction: float
    current_model_attributed_conversions_value: float
    current_model_attributed_conversions_value_per_cost: float
    engagement_rate: float
    engagements: int
    hotel_average_lead_value_micros: float
    hotel_commission_rate_micros: int
    hotel_expected_commission_cost: float
    hotel_price_difference_percentage: float
    hotel_eligible_impressions: int
    historical_creative_quality_score: QualityScoreBucketEnum.QualityScoreBucket
    historical_landing_page_quality_score: QualityScoreBucketEnum.QualityScoreBucket
    historical_quality_score: int
    historical_search_predicted_ctr: QualityScoreBucketEnum.QualityScoreBucket
    gmail_forwards: int
    gmail_saves: int
    gmail_secondary_clicks: int
    impressions_from_store_reach: int
    impressions: int
    interaction_rate: float
    interactions: int
    interaction_event_types: MutableSequence[
        InteractionEventTypeEnum.InteractionEventType
    ]
    invalid_click_rate: float
    invalid_clicks: int
    general_invalid_click_rate: float
    general_invalid_clicks: int
    message_chats: int
    message_impressions: int
    message_chat_rate: float
    mobile_friendly_clicks_percentage: float
    optimization_score_uplift: float
    optimization_score_url: str
    organic_clicks: int
    organic_clicks_per_query: float
    organic_impressions: int
    organic_impressions_per_query: float
    organic_queries: int
    percent_new_visitors: float
    phone_calls: int
    phone_impressions: int
    phone_through_rate: float
    relative_ctr: float
    search_absolute_top_impression_share: float
    search_budget_lost_absolute_top_impression_share: float
    search_budget_lost_impression_share: float
    search_budget_lost_top_impression_share: float
    search_click_share: float
    search_exact_match_impression_share: float
    search_impression_share: float
    search_rank_lost_absolute_top_impression_share: float
    search_rank_lost_impression_share: float
    search_rank_lost_top_impression_share: float
    search_top_impression_share: float
    search_volume: SearchVolumeRange
    speed_score: int
    average_target_cpa_micros: int
    average_target_roas: float
    top_impression_percentage: float
    valid_accelerated_mobile_pages_clicks_percentage: float
    value_per_all_conversions: float
    value_per_all_conversions_by_conversion_date: float
    value_per_conversion: float
    value_per_conversions_by_conversion_date: float
    value_per_current_model_attributed_conversion: float
    video_quartile_p100_rate: float
    video_quartile_p25_rate: float
    video_quartile_p50_rate: float
    video_quartile_p75_rate: float
    video_trueview_view_rate: float
    video_trueview_views: int
    view_through_conversions: int
    sk_ad_network_installs: int
    sk_ad_network_total_conversions: int
    publisher_purchased_clicks: int
    publisher_organic_clicks: int
    publisher_unknown_clicks: int
    all_conversions_from_location_asset_click_to_call: float
    all_conversions_from_location_asset_directions: float
    all_conversions_from_location_asset_menu: float
    all_conversions_from_location_asset_order: float
    all_conversions_from_location_asset_other_engagement: float
    all_conversions_from_location_asset_store_visits: float
    all_conversions_from_location_asset_website: float
    eligible_impressions_from_location_asset_store_reach: int
    view_through_conversions_from_location_asset_click_to_call: float
    view_through_conversions_from_location_asset_directions: float
    view_through_conversions_from_location_asset_menu: float
    view_through_conversions_from_location_asset_order: float
    view_through_conversions_from_location_asset_other_engagement: float
    view_through_conversions_from_location_asset_store_visits: float
    view_through_conversions_from_location_asset_website: float
    orders: float
    average_order_value_micros: int
    average_cart_size: float
    cost_of_goods_sold_micros: int
    gross_profit_micros: int
    gross_profit_margin: float
    revenue_micros: int
    units_sold: float
    cross_sell_cost_of_goods_sold_micros: int
    cross_sell_gross_profit_micros: int
    cross_sell_revenue_micros: int
    cross_sell_units_sold: float
    lead_cost_of_goods_sold_micros: int
    lead_gross_profit_micros: int
    lead_revenue_micros: int
    lead_units_sold: float
    unique_users: int
    average_impression_frequency_per_user: float
    linked_entities_count: int
    linked_sample_entities: MutableSequence[str]
    asset_pinned_total_count: int
    asset_pinned_as_headline_position_one_count: int
    asset_pinned_as_headline_position_two_count: int
    asset_pinned_as_headline_position_three_count: int
    asset_pinned_as_description_position_one_count: int
    asset_pinned_as_description_position_two_count: int
    store_visits_last_click_model_attributed_conversions: float
    results_conversions_purchase: float
    video_trueview_view_rate_in_feed: float
    video_trueview_view_rate_in_stream: float
    video_trueview_view_rate_shorts: float
    coviewed_impressions: int
    primary_impressions: int
    platform_comparable_conversions_from_interactions_rate: float
    platform_comparable_conversions: float
    platform_comparable_conversions_value: float
    platform_comparable_conversions_value_per_cost: float
    platform_comparable_conversions_by_conversion_date: float
    platform_comparable_conversions_value_by_conversion_date: float
    platform_comparable_conversions_from_interactions_value_per_interaction: float
    cost_per_platform_comparable_conversion: float
    value_per_platform_comparable_conversion: float
    value_per_platform_comparable_conversions_by_conversion_date: float
    cost_converted_currency_per_platform_comparable_conversion: float
    value_adjustment: float
    all_value_adjustment: float
    clicks_unique_query_clusters: int
    conversions_unique_query_clusters: int
    impressions_unique_query_clusters: int
    video_watch_time_duration_millis: int
    average_video_watch_time_duration_millis: int
    svr: int
    active_view_audibility_measurable_impressions: int
    active_view_audibility_measurable_impressions_rate: float
    active_view_audibility_invalid_measurable_impressions_rate: float
    active_view_audibility_invalid_givt_measurable_impressions_rate: float
    active_view_audible_impressions: int
    active_view_audible_impressions_rate: float
    active_view_audible_two_seconds_impressions: int
    active_view_audible_two_seconds_impressions_rate: float
    active_view_audible_thirty_seconds_impressions: int
    active_view_audible_thirty_seconds_impressions_rate: float
    active_view_audible_quartile_p25_rate: float
    active_view_audible_quartile_p50_rate: float
    active_view_audible_quartile_p75_rate: float
    active_view_audible_quartile_p100_rate: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        absolute_top_impression_percentage: float = ...,
        active_view_cpm: float = ...,
        active_view_ctr: float = ...,
        active_view_impressions: int = ...,
        active_view_measurability: float = ...,
        active_view_measurable_cost_micros: int = ...,
        active_view_measurable_impressions: int = ...,
        active_view_viewability: float = ...,
        all_conversions_from_interactions_rate: float = ...,
        all_conversions_value: float = ...,
        all_conversions_value_by_conversion_date: float = ...,
        all_new_customer_lifetime_value: float = ...,
        all_conversions: float = ...,
        all_conversions_by_conversion_date: float = ...,
        all_conversions_value_per_cost: float = ...,
        all_conversions_from_click_to_call: float = ...,
        all_conversions_from_directions: float = ...,
        all_conversions_from_interactions_value_per_interaction: float = ...,
        all_conversions_from_menu: float = ...,
        all_conversions_from_order: float = ...,
        all_conversions_from_other_engagement: float = ...,
        all_conversions_from_store_visit: float = ...,
        all_conversions_from_store_website: float = ...,
        auction_insight_search_absolute_top_impression_percentage: float = ...,
        auction_insight_search_impression_share: float = ...,
        auction_insight_search_outranking_share: float = ...,
        auction_insight_search_overlap_rate: float = ...,
        auction_insight_search_position_above_rate: float = ...,
        auction_insight_search_top_impression_percentage: float = ...,
        average_cost: float = ...,
        average_cpc: float = ...,
        average_cpe: float = ...,
        average_cpm: float = ...,
        trueview_average_cpv: float = ...,
        average_page_views: float = ...,
        average_time_on_site: float = ...,
        benchmark_average_max_cpc: float = ...,
        biddable_app_install_conversions: float = ...,
        biddable_app_post_install_conversions: float = ...,
        biddable_cohort_app_post_install_conversions: float = ...,
        benchmark_ctr: float = ...,
        bounce_rate: float = ...,
        clicks: int = ...,
        combined_clicks: int = ...,
        combined_clicks_per_query: float = ...,
        combined_queries: int = ...,
        content_budget_lost_impression_share: float = ...,
        content_impression_share: float = ...,
        conversion_last_received_request_date_time: str = ...,
        conversion_last_conversion_date: str = ...,
        content_rank_lost_impression_share: float = ...,
        conversions_from_interactions_rate: float = ...,
        conversions_value: float = ...,
        conversions_value_by_conversion_date: float = ...,
        new_customer_lifetime_value: float = ...,
        conversions_value_per_cost: float = ...,
        conversions_from_interactions_value_per_interaction: float = ...,
        conversions: float = ...,
        conversions_by_conversion_date: float = ...,
        cost_micros: int = ...,
        cost_per_all_conversions: float = ...,
        cost_per_conversion: float = ...,
        cost_per_current_model_attributed_conversion: float = ...,
        cross_device_conversions: float = ...,
        cross_device_conversions_by_conversion_date: float = ...,
        cross_device_conversions_value: float = ...,
        cross_device_conversions_value_micros: int = ...,
        cross_device_conversions_value_by_conversion_date: float = ...,
        ctr: float = ...,
        current_model_attributed_conversions: float = ...,
        current_model_attributed_conversions_from_interactions_rate: float = ...,
        current_model_attributed_conversions_from_interactions_value_per_interaction: float = ...,
        current_model_attributed_conversions_value: float = ...,
        current_model_attributed_conversions_value_per_cost: float = ...,
        engagement_rate: float = ...,
        engagements: int = ...,
        hotel_average_lead_value_micros: float = ...,
        hotel_commission_rate_micros: int = ...,
        hotel_expected_commission_cost: float = ...,
        hotel_price_difference_percentage: float = ...,
        hotel_eligible_impressions: int = ...,
        historical_creative_quality_score: QualityScoreBucketEnum.QualityScoreBucket = ...,
        historical_landing_page_quality_score: QualityScoreBucketEnum.QualityScoreBucket = ...,
        historical_quality_score: int = ...,
        historical_search_predicted_ctr: QualityScoreBucketEnum.QualityScoreBucket = ...,
        gmail_forwards: int = ...,
        gmail_saves: int = ...,
        gmail_secondary_clicks: int = ...,
        impressions_from_store_reach: int = ...,
        impressions: int = ...,
        interaction_rate: float = ...,
        interactions: int = ...,
        interaction_event_types: MutableSequence[
            InteractionEventTypeEnum.InteractionEventType
        ] = ...,
        invalid_click_rate: float = ...,
        invalid_clicks: int = ...,
        general_invalid_click_rate: float = ...,
        general_invalid_clicks: int = ...,
        message_chats: int = ...,
        message_impressions: int = ...,
        message_chat_rate: float = ...,
        mobile_friendly_clicks_percentage: float = ...,
        optimization_score_uplift: float = ...,
        optimization_score_url: str = ...,
        organic_clicks: int = ...,
        organic_clicks_per_query: float = ...,
        organic_impressions: int = ...,
        organic_impressions_per_query: float = ...,
        organic_queries: int = ...,
        percent_new_visitors: float = ...,
        phone_calls: int = ...,
        phone_impressions: int = ...,
        phone_through_rate: float = ...,
        relative_ctr: float = ...,
        search_absolute_top_impression_share: float = ...,
        search_budget_lost_absolute_top_impression_share: float = ...,
        search_budget_lost_impression_share: float = ...,
        search_budget_lost_top_impression_share: float = ...,
        search_click_share: float = ...,
        search_exact_match_impression_share: float = ...,
        search_impression_share: float = ...,
        search_rank_lost_absolute_top_impression_share: float = ...,
        search_rank_lost_impression_share: float = ...,
        search_rank_lost_top_impression_share: float = ...,
        search_top_impression_share: float = ...,
        search_volume: SearchVolumeRange = ...,
        speed_score: int = ...,
        average_target_cpa_micros: int = ...,
        average_target_roas: float = ...,
        top_impression_percentage: float = ...,
        valid_accelerated_mobile_pages_clicks_percentage: float = ...,
        value_per_all_conversions: float = ...,
        value_per_all_conversions_by_conversion_date: float = ...,
        value_per_conversion: float = ...,
        value_per_conversions_by_conversion_date: float = ...,
        value_per_current_model_attributed_conversion: float = ...,
        video_quartile_p100_rate: float = ...,
        video_quartile_p25_rate: float = ...,
        video_quartile_p50_rate: float = ...,
        video_quartile_p75_rate: float = ...,
        video_trueview_view_rate: float = ...,
        video_trueview_views: int = ...,
        view_through_conversions: int = ...,
        sk_ad_network_installs: int = ...,
        sk_ad_network_total_conversions: int = ...,
        publisher_purchased_clicks: int = ...,
        publisher_organic_clicks: int = ...,
        publisher_unknown_clicks: int = ...,
        all_conversions_from_location_asset_click_to_call: float = ...,
        all_conversions_from_location_asset_directions: float = ...,
        all_conversions_from_location_asset_menu: float = ...,
        all_conversions_from_location_asset_order: float = ...,
        all_conversions_from_location_asset_other_engagement: float = ...,
        all_conversions_from_location_asset_store_visits: float = ...,
        all_conversions_from_location_asset_website: float = ...,
        eligible_impressions_from_location_asset_store_reach: int = ...,
        view_through_conversions_from_location_asset_click_to_call: float = ...,
        view_through_conversions_from_location_asset_directions: float = ...,
        view_through_conversions_from_location_asset_menu: float = ...,
        view_through_conversions_from_location_asset_order: float = ...,
        view_through_conversions_from_location_asset_other_engagement: float = ...,
        view_through_conversions_from_location_asset_store_visits: float = ...,
        view_through_conversions_from_location_asset_website: float = ...,
        orders: float = ...,
        average_order_value_micros: int = ...,
        average_cart_size: float = ...,
        cost_of_goods_sold_micros: int = ...,
        gross_profit_micros: int = ...,
        gross_profit_margin: float = ...,
        revenue_micros: int = ...,
        units_sold: float = ...,
        cross_sell_cost_of_goods_sold_micros: int = ...,
        cross_sell_gross_profit_micros: int = ...,
        cross_sell_revenue_micros: int = ...,
        cross_sell_units_sold: float = ...,
        lead_cost_of_goods_sold_micros: int = ...,
        lead_gross_profit_micros: int = ...,
        lead_revenue_micros: int = ...,
        lead_units_sold: float = ...,
        unique_users: int = ...,
        average_impression_frequency_per_user: float = ...,
        linked_entities_count: int = ...,
        linked_sample_entities: MutableSequence[str] = ...,
        asset_pinned_total_count: int = ...,
        asset_pinned_as_headline_position_one_count: int = ...,
        asset_pinned_as_headline_position_two_count: int = ...,
        asset_pinned_as_headline_position_three_count: int = ...,
        asset_pinned_as_description_position_one_count: int = ...,
        asset_pinned_as_description_position_two_count: int = ...,
        store_visits_last_click_model_attributed_conversions: float = ...,
        results_conversions_purchase: float = ...,
        video_trueview_view_rate_in_feed: float = ...,
        video_trueview_view_rate_in_stream: float = ...,
        video_trueview_view_rate_shorts: float = ...,
        coviewed_impressions: int = ...,
        primary_impressions: int = ...,
        platform_comparable_conversions_from_interactions_rate: float = ...,
        platform_comparable_conversions: float = ...,
        platform_comparable_conversions_value: float = ...,
        platform_comparable_conversions_value_per_cost: float = ...,
        platform_comparable_conversions_by_conversion_date: float = ...,
        platform_comparable_conversions_value_by_conversion_date: float = ...,
        platform_comparable_conversions_from_interactions_value_per_interaction: float = ...,
        cost_per_platform_comparable_conversion: float = ...,
        value_per_platform_comparable_conversion: float = ...,
        value_per_platform_comparable_conversions_by_conversion_date: float = ...,
        cost_converted_currency_per_platform_comparable_conversion: float = ...,
        value_adjustment: float = ...,
        all_value_adjustment: float = ...,
        clicks_unique_query_clusters: int = ...,
        conversions_unique_query_clusters: int = ...,
        impressions_unique_query_clusters: int = ...,
        video_watch_time_duration_millis: int = ...,
        average_video_watch_time_duration_millis: int = ...,
        svr: int = ...,
        active_view_audibility_measurable_impressions: int = ...,
        active_view_audibility_measurable_impressions_rate: float = ...,
        active_view_audibility_invalid_measurable_impressions_rate: float = ...,
        active_view_audibility_invalid_givt_measurable_impressions_rate: float = ...,
        active_view_audible_impressions: int = ...,
        active_view_audible_impressions_rate: float = ...,
        active_view_audible_two_seconds_impressions: int = ...,
        active_view_audible_two_seconds_impressions_rate: float = ...,
        active_view_audible_thirty_seconds_impressions: int = ...,
        active_view_audible_thirty_seconds_impressions_rate: float = ...,
        active_view_audible_quartile_p25_rate: float = ...,
        active_view_audible_quartile_p50_rate: float = ...,
        active_view_audible_quartile_p75_rate: float = ...,
        active_view_audible_quartile_p100_rate: float = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "absolute_top_impression_percentage",
            "active_view_cpm",
            "active_view_ctr",
            "active_view_impressions",
            "active_view_measurability",
            "active_view_measurable_cost_micros",
            "active_view_measurable_impressions",
            "active_view_viewability",
            "all_conversions_from_interactions_rate",
            "all_conversions_value",
            "all_conversions_value_by_conversion_date",
            "all_new_customer_lifetime_value",
            "all_conversions",
            "all_conversions_by_conversion_date",
            "all_conversions_value_per_cost",
            "all_conversions_from_click_to_call",
            "all_conversions_from_directions",
            "all_conversions_from_interactions_value_per_interaction",
            "all_conversions_from_menu",
            "all_conversions_from_order",
            "all_conversions_from_other_engagement",
            "all_conversions_from_store_visit",
            "all_conversions_from_store_website",
            "auction_insight_search_absolute_top_impression_percentage",
            "auction_insight_search_impression_share",
            "auction_insight_search_outranking_share",
            "auction_insight_search_overlap_rate",
            "auction_insight_search_position_above_rate",
            "auction_insight_search_top_impression_percentage",
            "average_cost",
            "average_cpc",
            "average_cpe",
            "average_cpm",
            "trueview_average_cpv",
            "average_page_views",
            "average_time_on_site",
            "benchmark_average_max_cpc",
            "biddable_app_install_conversions",
            "biddable_app_post_install_conversions",
            "biddable_cohort_app_post_install_conversions",
            "benchmark_ctr",
            "bounce_rate",
            "clicks",
            "combined_clicks",
            "combined_clicks_per_query",
            "combined_queries",
            "content_budget_lost_impression_share",
            "content_impression_share",
            "conversion_last_received_request_date_time",
            "conversion_last_conversion_date",
            "content_rank_lost_impression_share",
            "conversions_from_interactions_rate",
            "conversions_value",
            "conversions_value_by_conversion_date",
            "new_customer_lifetime_value",
            "conversions_value_per_cost",
            "conversions_from_interactions_value_per_interaction",
            "conversions",
            "conversions_by_conversion_date",
            "cost_micros",
            "cost_per_all_conversions",
            "cost_per_conversion",
            "cost_per_current_model_attributed_conversion",
            "cross_device_conversions",
            "cross_device_conversions_by_conversion_date",
            "cross_device_conversions_value",
            "cross_device_conversions_value_micros",
            "cross_device_conversions_value_by_conversion_date",
            "ctr",
            "current_model_attributed_conversions",
            "current_model_attributed_conversions_from_interactions_rate",
            "current_model_attributed_conversions_from_interactions_value_per_interaction",
            "current_model_attributed_conversions_value",
            "current_model_attributed_conversions_value_per_cost",
            "engagement_rate",
            "engagements",
            "hotel_average_lead_value_micros",
            "hotel_commission_rate_micros",
            "hotel_expected_commission_cost",
            "hotel_price_difference_percentage",
            "hotel_eligible_impressions",
            "historical_creative_quality_score",
            "historical_landing_page_quality_score",
            "historical_quality_score",
            "historical_search_predicted_ctr",
            "gmail_forwards",
            "gmail_saves",
            "gmail_secondary_clicks",
            "impressions_from_store_reach",
            "impressions",
            "interaction_rate",
            "interactions",
            "interaction_event_types",
            "invalid_click_rate",
            "invalid_clicks",
            "general_invalid_click_rate",
            "general_invalid_clicks",
            "message_chats",
            "message_impressions",
            "message_chat_rate",
            "mobile_friendly_clicks_percentage",
            "optimization_score_uplift",
            "optimization_score_url",
            "organic_clicks",
            "organic_clicks_per_query",
            "organic_impressions",
            "organic_impressions_per_query",
            "organic_queries",
            "percent_new_visitors",
            "phone_calls",
            "phone_impressions",
            "phone_through_rate",
            "relative_ctr",
            "search_absolute_top_impression_share",
            "search_budget_lost_absolute_top_impression_share",
            "search_budget_lost_impression_share",
            "search_budget_lost_top_impression_share",
            "search_click_share",
            "search_exact_match_impression_share",
            "search_impression_share",
            "search_rank_lost_absolute_top_impression_share",
            "search_rank_lost_impression_share",
            "search_rank_lost_top_impression_share",
            "search_top_impression_share",
            "search_volume",
            "speed_score",
            "average_target_cpa_micros",
            "average_target_roas",
            "top_impression_percentage",
            "valid_accelerated_mobile_pages_clicks_percentage",
            "value_per_all_conversions",
            "value_per_all_conversions_by_conversion_date",
            "value_per_conversion",
            "value_per_conversions_by_conversion_date",
            "value_per_current_model_attributed_conversion",
            "video_quartile_p100_rate",
            "video_quartile_p25_rate",
            "video_quartile_p50_rate",
            "video_quartile_p75_rate",
            "video_trueview_view_rate",
            "video_trueview_views",
            "view_through_conversions",
            "sk_ad_network_installs",
            "sk_ad_network_total_conversions",
            "publisher_purchased_clicks",
            "publisher_organic_clicks",
            "publisher_unknown_clicks",
            "all_conversions_from_location_asset_click_to_call",
            "all_conversions_from_location_asset_directions",
            "all_conversions_from_location_asset_menu",
            "all_conversions_from_location_asset_order",
            "all_conversions_from_location_asset_other_engagement",
            "all_conversions_from_location_asset_store_visits",
            "all_conversions_from_location_asset_website",
            "eligible_impressions_from_location_asset_store_reach",
            "view_through_conversions_from_location_asset_click_to_call",
            "view_through_conversions_from_location_asset_directions",
            "view_through_conversions_from_location_asset_menu",
            "view_through_conversions_from_location_asset_order",
            "view_through_conversions_from_location_asset_other_engagement",
            "view_through_conversions_from_location_asset_store_visits",
            "view_through_conversions_from_location_asset_website",
            "orders",
            "average_order_value_micros",
            "average_cart_size",
            "cost_of_goods_sold_micros",
            "gross_profit_micros",
            "gross_profit_margin",
            "revenue_micros",
            "units_sold",
            "cross_sell_cost_of_goods_sold_micros",
            "cross_sell_gross_profit_micros",
            "cross_sell_revenue_micros",
            "cross_sell_units_sold",
            "lead_cost_of_goods_sold_micros",
            "lead_gross_profit_micros",
            "lead_revenue_micros",
            "lead_units_sold",
            "unique_users",
            "average_impression_frequency_per_user",
            "linked_entities_count",
            "linked_sample_entities",
            "asset_pinned_total_count",
            "asset_pinned_as_headline_position_one_count",
            "asset_pinned_as_headline_position_two_count",
            "asset_pinned_as_headline_position_three_count",
            "asset_pinned_as_description_position_one_count",
            "asset_pinned_as_description_position_two_count",
            "store_visits_last_click_model_attributed_conversions",
            "results_conversions_purchase",
            "video_trueview_view_rate_in_feed",
            "video_trueview_view_rate_in_stream",
            "video_trueview_view_rate_shorts",
            "coviewed_impressions",
            "primary_impressions",
            "platform_comparable_conversions_from_interactions_rate",
            "platform_comparable_conversions",
            "platform_comparable_conversions_value",
            "platform_comparable_conversions_value_per_cost",
            "platform_comparable_conversions_by_conversion_date",
            "platform_comparable_conversions_value_by_conversion_date",
            "platform_comparable_conversions_from_interactions_value_per_interaction",
            "cost_per_platform_comparable_conversion",
            "value_per_platform_comparable_conversion",
            "value_per_platform_comparable_conversions_by_conversion_date",
            "cost_converted_currency_per_platform_comparable_conversion",
            "value_adjustment",
            "all_value_adjustment",
            "clicks_unique_query_clusters",
            "conversions_unique_query_clusters",
            "impressions_unique_query_clusters",
            "video_watch_time_duration_millis",
            "average_video_watch_time_duration_millis",
            "svr",
            "active_view_audibility_measurable_impressions",
            "active_view_audibility_measurable_impressions_rate",
            "active_view_audibility_invalid_measurable_impressions_rate",
            "active_view_audibility_invalid_givt_measurable_impressions_rate",
            "active_view_audible_impressions",
            "active_view_audible_impressions_rate",
            "active_view_audible_two_seconds_impressions",
            "active_view_audible_two_seconds_impressions_rate",
            "active_view_audible_thirty_seconds_impressions",
            "active_view_audible_thirty_seconds_impressions_rate",
            "active_view_audible_quartile_p25_rate",
            "active_view_audible_quartile_p50_rate",
            "active_view_audible_quartile_p75_rate",
            "active_view_audible_quartile_p100_rate",
        ],
    ) -> bool: ...

class SearchVolumeRange(proto.Message):
    min_: int
    max_: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        min_: int = ...,
        max_: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["min_", "max_"]
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from google.rpc.status_pb2 import Status
from typing_extensions import Literal

from google.ads.googleads.v23.common.types.policy import PolicyViolationKey
from google.ads.googleads.v23.enums.types.response_content_type import (
    ResponseContentTypeEnum,
)
from google.ads.googleads.v23.resources.types.asset_group_signal import AssetGroupSignal

_M = TypeVar("_M")

class AssetGroupSignalOperation(proto.Message):
    exempt_policy_violation_keys: MutableSequence[PolicyViolationKey]
    create: AssetGroupSignal
    remove: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        exempt_policy_violation_keys: MutableSequence[PolicyViolationKey] = ...,
        create: AssetGroupSignal = ...,
        remove: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["exempt_policy_violation_keys", "create", "remove"]
    ) -> bool: ...

class MutateAssetGroupSignalResult(proto.Message):
    resource_name: str
    asset_group_signal: AssetGroupSignal
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        asset_group_signal: AssetGroupSignal = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name", "asset_group_signal"]
    ) -> bool: ...

class MutateAssetGroupSignalsRequest(proto.Message):
    customer_id: str
    operations: MutableSequence[AssetGroupSignalOperation]
    partial_failure: bool
    validate_only: bool
    response_content_type: ResponseContentTypeEnum.ResponseContentType
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        operations: MutableSequence[AssetGroupSignalOperation] = ...,
        partial_failure: bool = ...,
        validate_only: bool = ...,
        response_content_type: ResponseContentTypeEnum.ResponseContentType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "operations",
            "partial_failure",
            "validate_only",
            "response_content_type",
        ],
    ) -> bool: ...

class MutateAssetGroupSignalsResponse(proto.Message):
    results: MutableSequence[MutateAssetGroupSignalResult]
    partial_failure_error: Status
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        results: MutableSequence[MutateAssetGroupSignalResult] = ...,
        partial_failure_error: Status = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["results", "partial_failure_error"]
    ) -> bool: ...



from .base import (
    BiddingSeasonalityAdjustmentServiceTransport as BiddingSeasonalityAdjustmentServiceTransport,
)
from .grpc import (
    BiddingSeasonalityAdjustmentServiceGrpcTransport as BiddingSeasonalityAdjustmentServiceGrpcTransport,
)
from .grpc_asyncio import (
    BiddingSeasonalityAdjustmentServiceGrpcAsyncIOTransport as BiddingSeasonalityAdjustmentServiceGrpcAsyncIOTransport,
)

__all__ = [
    "BiddingSeasonalityAdjustmentServiceTransport",
    "BiddingSeasonalityAdjustmentServiceGrpcTransport",
    "BiddingSeasonalityAdjustmentServiceGrpcAsyncIOTransport",
]



from .types.accessible_bidding_strategy import (
    AccessibleBiddingStrategy as AccessibleBiddingStrategy,
)
from .types.account_budget import AccountBudget as AccountBudget
from .types.account_budget_proposal import (
    AccountBudgetProposal as AccountBudgetProposal,
)
from .types.account_link import (
    AccountLink as AccountLink,
    ThirdPartyAppAnalyticsLinkIdentifier as ThirdPartyAppAnalyticsLinkIdentifier,
)
from .types.ad import Ad as Ad
from .types.ad_group import AdGroup as AdGroup
from .types.ad_group_ad import (
    AdGroupAd as AdGroupAd,
    AdGroupAdAssetAutomationSetting as AdGroupAdAssetAutomationSetting,
    AdGroupAdPolicySummary as AdGroupAdPolicySummary,
)
from .types.ad_group_ad_asset_combination_view import (
    AdGroupAdAssetCombinationView as AdGroupAdAssetCombinationView,
)
from .types.ad_group_ad_asset_view import (
    AdGroupAdAssetPolicySummary as AdGroupAdAssetPolicySummary,
    AdGroupAdAssetView as AdGroupAdAssetView,
)
from .types.ad_group_ad_label import AdGroupAdLabel as AdGroupAdLabel
from .types.ad_group_asset import AdGroupAsset as AdGroupAsset
from .types.ad_group_asset_set import AdGroupAssetSet as AdGroupAssetSet
from .types.ad_group_audience_view import AdGroupAudienceView as AdGroupAudienceView
from .types.ad_group_bid_modifier import AdGroupBidModifier as AdGroupBidModifier
from .types.ad_group_criterion import AdGroupCriterion as AdGroupCriterion
from .types.ad_group_criterion_customizer import (
    AdGroupCriterionCustomizer as AdGroupCriterionCustomizer,
)
from .types.ad_group_criterion_label import (
    AdGroupCriterionLabel as AdGroupCriterionLabel,
)
from .types.ad_group_criterion_simulation import (
    AdGroupCriterionSimulation as AdGroupCriterionSimulation,
)
from .types.ad_group_customizer import AdGroupCustomizer as AdGroupCustomizer
from .types.ad_group_label import AdGroupLabel as AdGroupLabel
from .types.ad_group_simulation import AdGroupSimulation as AdGroupSimulation
from .types.ad_parameter import AdParameter as AdParameter
from .types.ad_schedule_view import AdScheduleView as AdScheduleView
from .types.age_range_view import AgeRangeView as AgeRangeView
from .types.ai_max_search_term_ad_combination_view import (
    AiMaxSearchTermAdCombinationView as AiMaxSearchTermAdCombinationView,
)
from .types.android_privacy_shared_key_google_ad_group import (
    AndroidPrivacySharedKeyGoogleAdGroup as AndroidPrivacySharedKeyGoogleAdGroup,
)
from .types.android_privacy_shared_key_google_campaign import (
    AndroidPrivacySharedKeyGoogleCampaign as AndroidPrivacySharedKeyGoogleCampaign,
)
from .types.android_privacy_shared_key_google_network_type import (
    AndroidPrivacySharedKeyGoogleNetworkType as AndroidPrivacySharedKeyGoogleNetworkType,
)
from .types.asset import (
    Asset as Asset,
    AssetFieldTypePolicySummary as AssetFieldTypePolicySummary,
    AssetPolicySummary as AssetPolicySummary,
)
from .types.asset_field_type_view import AssetFieldTypeView as AssetFieldTypeView
from .types.asset_group import (
    AdStrengthActionItem as AdStrengthActionItem,
    AssetCoverage as AssetCoverage,
    AssetGroup as AssetGroup,
)
from .types.asset_group_asset import AssetGroupAsset as AssetGroupAsset
from .types.asset_group_listing_group_filter import (
    AssetGroupListingGroupFilter as AssetGroupListingGroupFilter,
    ListingGroupFilterDimension as ListingGroupFilterDimension,
    ListingGroupFilterDimensionPath as ListingGroupFilterDimensionPath,
)
from .types.asset_group_product_group_view import (
    AssetGroupProductGroupView as AssetGroupProductGroupView,
)
from .types.asset_group_signal import AssetGroupSignal as AssetGroupSignal
from .types.asset_group_top_combination_view import (
    AssetGroupAssetCombinationData as AssetGroupAssetCombinationData,
    AssetGroupTopCombinationView as AssetGroupTopCombinationView,
)
from .types.asset_set import AssetSet as AssetSet
from .types.asset_set_asset import AssetSetAsset as AssetSetAsset
from .types.asset_set_type_view import AssetSetTypeView as AssetSetTypeView
from .types.audience import Audience as Audience
from .types.batch_job import BatchJob as BatchJob
from .types.bidding_data_exclusion import BiddingDataExclusion as BiddingDataExclusion
from .types.bidding_seasonality_adjustment import (
    BiddingSeasonalityAdjustment as BiddingSeasonalityAdjustment,
)
from .types.bidding_strategy import BiddingStrategy as BiddingStrategy
from .types.bidding_strategy_simulation import (
    BiddingStrategySimulation as BiddingStrategySimulation,
)
from .types.billing_setup import BillingSetup as BillingSetup
from .types.call_view import CallView as CallView
from .types.campaign import Campaign as Campaign
from .types.campaign_aggregate_asset_view import (
    CampaignAggregateAssetView as CampaignAggregateAssetView,
)
from .types.campaign_asset import CampaignAsset as CampaignAsset
from .types.campaign_asset_set import CampaignAssetSet as CampaignAssetSet
from .types.campaign_audience_view import CampaignAudienceView as CampaignAudienceView
from .types.campaign_bid_modifier import CampaignBidModifier as CampaignBidModifier
from .types.campaign_budget import CampaignBudget as CampaignBudget
from .types.campaign_conversion_goal import (
    CampaignConversionGoal as CampaignConversionGoal,
)
from .types.campaign_criterion import CampaignCriterion as CampaignCriterion
from .types.campaign_customizer import CampaignCustomizer as CampaignCustomizer
from .types.campaign_draft import CampaignDraft as CampaignDraft
from .types.campaign_group import CampaignGroup as CampaignGroup
from .types.campaign_label import CampaignLabel as CampaignLabel
from .types.campaign_lifecycle_goal import (
    CampaignLifecycleGoal as CampaignLifecycleGoal,
    CustomerAcquisitionGoalSettings as CustomerAcquisitionGoalSettings,
)
from .types.campaign_search_term_insight import (
    CampaignSearchTermInsight as CampaignSearchTermInsight,
)
from .types.campaign_search_term_view import (
    CampaignSearchTermView as CampaignSearchTermView,
)
from .types.campaign_shared_set import CampaignSharedSet as CampaignSharedSet
from .types.campaign_simulation import CampaignSimulation as CampaignSimulation
from .types.carrier_constant import CarrierConstant as CarrierConstant
from .types.change_event import ChangeEvent as ChangeEvent
from .types.change_status import ChangeStatus as ChangeStatus
from .types.channel_aggregate_asset_view import (
    ChannelAggregateAssetView as ChannelAggregateAssetView,
)
from .types.click_view import ClickView as ClickView
from .types.combined_audience import CombinedAudience as CombinedAudience
from .types.content_criterion_view import ContentCriterionView as ContentCriterionView
from .types.conversion_action import ConversionAction as ConversionAction
from .types.conversion_custom_variable import (
    ConversionCustomVariable as ConversionCustomVariable,
)
from .types.conversion_goal_campaign_config import (
    ConversionGoalCampaignConfig as ConversionGoalCampaignConfig,
)
from .types.conversion_value_rule import ConversionValueRule as ConversionValueRule
from .types.conversion_value_rule_set import (
    ConversionValueRuleSet as ConversionValueRuleSet,
)
from .types.currency_constant import CurrencyConstant as CurrencyConstant
from .types.custom_audience import (
    CustomAudience as CustomAudience,
    CustomAudienceMember as CustomAudienceMember,
)
from .types.custom_conversion_goal import CustomConversionGoal as CustomConversionGoal
from .types.custom_interest import (
    CustomInterest as CustomInterest,
    CustomInterestMember as CustomInterestMember,
)
from .types.customer import (
    CallReportingSetting as CallReportingSetting,
    ConversionTrackingSetting as ConversionTrackingSetting,
    Customer as Customer,
    CustomerAgreementSetting as CustomerAgreementSetting,
    GranularInsuranceStatus as GranularInsuranceStatus,
    GranularLicenseStatus as GranularLicenseStatus,
    LocalServicesSettings as LocalServicesSettings,
    RemarketingSetting as RemarketingSetting,
    VideoCustomer as VideoCustomer,
)
from .types.customer_asset import CustomerAsset as CustomerAsset
from .types.customer_asset_set import CustomerAssetSet as CustomerAssetSet
from .types.customer_client import CustomerClient as CustomerClient
from .types.customer_client_link import CustomerClientLink as CustomerClientLink
from .types.customer_conversion_goal import (
    CustomerConversionGoal as CustomerConversionGoal,
)
from .types.customer_customizer import CustomerCustomizer as CustomerCustomizer
from .types.customer_label import CustomerLabel as CustomerLabel
from .types.customer_lifecycle_goal import (
    CustomerLifecycleGoal as CustomerLifecycleGoal,
)
from .types.customer_manager_link import CustomerManagerLink as CustomerManagerLink
from .types.customer_negative_criterion import (
    CustomerNegativeCriterion as CustomerNegativeCriterion,
)
from .types.customer_search_term_insight import (
    CustomerSearchTermInsight as CustomerSearchTermInsight,
)
from .types.customer_sk_ad_network_conversion_value_schema import (
    CustomerSkAdNetworkConversionValueSchema as CustomerSkAdNetworkConversionValueSchema,
)
from .types.customer_user_access import CustomerUserAccess as CustomerUserAccess
from .types.customer_user_access_invitation import (
    CustomerUserAccessInvitation as CustomerUserAccessInvitation,
)
from .types.customizer_attribute import CustomizerAttribute as CustomizerAttribute
from .types.data_link import (
    DataLink as DataLink,
    YoutubeVideoIdentifier as YoutubeVideoIdentifier,
)
from .types.detail_content_suitability_placement_view import (
    DetailContentSuitabilityPlacementView as DetailContentSuitabilityPlacementView,
)
from .types.detail_placement_view import DetailPlacementView as DetailPlacementView
from .types.detailed_demographic import DetailedDemographic as DetailedDemographic
from .types.display_keyword_view import DisplayKeywordView as DisplayKeywordView
from .types.distance_view import DistanceView as DistanceView
from .types.domain_category import DomainCategory as DomainCategory
from .types.dynamic_search_ads_search_term_view import (
    DynamicSearchAdsSearchTermView as DynamicSearchAdsSearchTermView,
)
from .types.expanded_landing_page_view import (
    ExpandedLandingPageView as ExpandedLandingPageView,
)
from .types.experiment import Experiment as Experiment
from .types.experiment_arm import ExperimentArm as ExperimentArm
from .types.final_url_expansion_asset_view import (
    FinalUrlExpansionAssetView as FinalUrlExpansionAssetView,
)
from .types.gender_view import GenderView as GenderView
from .types.geo_target_constant import GeoTargetConstant as GeoTargetConstant
from .types.geographic_view import GeographicView as GeographicView
from .types.google_ads_field import GoogleAdsField as GoogleAdsField
from .types.group_content_suitability_placement_view import (
    GroupContentSuitabilityPlacementView as GroupContentSuitabilityPlacementView,
)
from .types.group_placement_view import GroupPlacementView as GroupPlacementView
from .types.hotel_group_view import HotelGroupView as HotelGroupView
from .types.hotel_performance_view import HotelPerformanceView as HotelPerformanceView
from .types.hotel_reconciliation import HotelReconciliation as HotelReconciliation
from .types.income_range_view import IncomeRangeView as IncomeRangeView
from .types.invoice import Invoice as Invoice
from .types.keyword_plan import (
    KeywordPlan as KeywordPlan,
    KeywordPlanForecastPeriod as KeywordPlanForecastPeriod,
)
from .types.keyword_plan_ad_group import KeywordPlanAdGroup as KeywordPlanAdGroup
from .types.keyword_plan_ad_group_keyword import (
    KeywordPlanAdGroupKeyword as KeywordPlanAdGroupKeyword,
)
from .types.keyword_plan_campaign import (
    KeywordPlanCampaign as KeywordPlanCampaign,
    KeywordPlanGeoTarget as KeywordPlanGeoTarget,
)
from .types.keyword_plan_campaign_keyword import (
    KeywordPlanCampaignKeyword as KeywordPlanCampaignKeyword,
)
from .types.keyword_theme_constant import KeywordThemeConstant as KeywordThemeConstant
from .types.keyword_view import KeywordView as KeywordView
from .types.label import Label as Label
from .types.landing_page_view import LandingPageView as LandingPageView
from .types.language_constant import LanguageConstant as LanguageConstant
from .types.lead_form_submission_data import (
    CustomLeadFormSubmissionField as CustomLeadFormSubmissionField,
    LeadFormSubmissionData as LeadFormSubmissionData,
    LeadFormSubmissionField as LeadFormSubmissionField,
)
from .types.life_event import LifeEvent as LifeEvent
from .types.local_services_employee import (
    Fellowship as Fellowship,
    LocalServicesEmployee as LocalServicesEmployee,
    Residency as Residency,
    UniversityDegree as UniversityDegree,
)
from .types.local_services_lead import (
    ContactDetails as ContactDetails,
    CreditDetails as CreditDetails,
    LocalServicesLead as LocalServicesLead,
    Note as Note,
)
from .types.local_services_lead_conversation import (
    LocalServicesLeadConversation as LocalServicesLeadConversation,
    MessageDetails as MessageDetails,
    PhoneCallDetails as PhoneCallDetails,
)
from .types.local_services_verification_artifact import (
    BackgroundCheckVerificationArtifact as BackgroundCheckVerificationArtifact,
    BusinessRegistrationCheckVerificationArtifact as BusinessRegistrationCheckVerificationArtifact,
    BusinessRegistrationDocument as BusinessRegistrationDocument,
    BusinessRegistrationNumber as BusinessRegistrationNumber,
    InsuranceVerificationArtifact as InsuranceVerificationArtifact,
    LicenseVerificationArtifact as LicenseVerificationArtifact,
    LocalServicesVerificationArtifact as LocalServicesVerificationArtifact,
)
from .types.location_interest_view import LocationInterestView as LocationInterestView
from .types.location_view import LocationView as LocationView
from .types.managed_placement_view import ManagedPlacementView as ManagedPlacementView
from .types.media_file import (
    MediaAudio as MediaAudio,
    MediaBundle as MediaBundle,
    MediaFile as MediaFile,
    MediaImage as MediaImage,
    MediaVideo as MediaVideo,
)
from .types.mobile_app_category_constant import (
    MobileAppCategoryConstant as MobileAppCategoryConstant,
)
from .types.mobile_device_constant import MobileDeviceConstant as MobileDeviceConstant
from .types.offline_conversion_upload_client_summary import (
    OfflineConversionAlert as OfflineConversionAlert,
    OfflineConversionError as OfflineConversionError,
    OfflineConversionSummary as OfflineConversionSummary,
    OfflineConversionUploadClientSummary as OfflineConversionUploadClientSummary,
)
from .types.offline_conversion_upload_conversion_action_summary import (
    OfflineConversionUploadConversionActionSummary as OfflineConversionUploadConversionActionSummary,
)
from .types.offline_user_data_job import (
    OfflineUserDataJob as OfflineUserDataJob,
    OfflineUserDataJobMetadata as OfflineUserDataJobMetadata,
)
from .types.operating_system_version_constant import (
    OperatingSystemVersionConstant as OperatingSystemVersionConstant,
)
from .types.paid_organic_search_term_view import (
    PaidOrganicSearchTermView as PaidOrganicSearchTermView,
)
from .types.parental_status_view import ParentalStatusView as ParentalStatusView
from .types.payments_account import PaymentsAccount as PaymentsAccount
from .types.per_store_view import PerStoreView as PerStoreView
from .types.performance_max_placement_view import (
    PerformanceMaxPlacementView as PerformanceMaxPlacementView,
)
from .types.product_category_constant import (
    ProductCategoryConstant as ProductCategoryConstant,
)
from .types.product_group_view import ProductGroupView as ProductGroupView
from .types.product_link import (
    AdvertisingPartnerIdentifier as AdvertisingPartnerIdentifier,
    DataPartnerIdentifier as DataPartnerIdentifier,
    GoogleAdsIdentifier as GoogleAdsIdentifier,
    MerchantCenterIdentifier as MerchantCenterIdentifier,
    ProductLink as ProductLink,
)
from .types.product_link_invitation import (
    AdvertisingPartnerLinkInvitationIdentifier as AdvertisingPartnerLinkInvitationIdentifier,
    HotelCenterLinkInvitationIdentifier as HotelCenterLinkInvitationIdentifier,
    MerchantCenterLinkInvitationIdentifier as MerchantCenterLinkInvitationIdentifier,
    ProductLinkInvitation as ProductLinkInvitation,
)
from .types.qualifying_question import QualifyingQuestion as QualifyingQuestion
from .types.recommendation import Recommendation as Recommendation
from .types.recommendation_subscription import (
    RecommendationSubscription as RecommendationSubscription,
)
from .types.remarketing_action import RemarketingAction as RemarketingAction
from .types.search_term_view import SearchTermView as SearchTermView
from .types.shared_criterion import SharedCriterion as SharedCriterion
from .types.shared_set import SharedSet as SharedSet
from .types.shopping_performance_view import (
    ShoppingPerformanceView as ShoppingPerformanceView,
)
from .types.shopping_product import ShoppingProduct as ShoppingProduct
from .types.smart_campaign_search_term_view import (
    SmartCampaignSearchTermView as SmartCampaignSearchTermView,
)
from .types.smart_campaign_setting import SmartCampaignSetting as SmartCampaignSetting
from .types.third_party_app_analytics_link import (
    ThirdPartyAppAnalyticsLink as ThirdPartyAppAnalyticsLink,
)
from .types.topic_constant import TopicConstant as TopicConstant
from .types.topic_view import TopicView as TopicView
from .types.travel_activity_group_view import (
    TravelActivityGroupView as TravelActivityGroupView,
)
from .types.travel_activity_performance_view import (
    TravelActivityPerformanceView as TravelActivityPerformanceView,
)
from .types.user_interest import UserInterest as UserInterest
from .types.user_list import UserList as UserList
from .types.user_list_customer_type import UserListCustomerType as UserListCustomerType
from .types.user_location_view import UserLocationView as UserLocationView
from .types.video import Video as Video
from .types.webpage_view import WebpageView as WebpageView

__all__ = [
    "AccessibleBiddingStrategy",
    "AccountBudget",
    "AccountBudgetProposal",
    "AccountLink",
    "Ad",
    "AdGroup",
    "AdGroupAd",
    "AdGroupAdAssetAutomationSetting",
    "AdGroupAdAssetCombinationView",
    "AdGroupAdAssetPolicySummary",
    "AdGroupAdAssetView",
    "AdGroupAdLabel",
    "AdGroupAdPolicySummary",
    "AdGroupAsset",
    "AdGroupAssetSet",
    "AdGroupAudienceView",
    "AdGroupBidModifier",
    "AdGroupCriterion",
    "AdGroupCriterionCustomizer",
    "AdGroupCriterionLabel",
    "AdGroupCriterionSimulation",
    "AdGroupCustomizer",
    "AdGroupLabel",
    "AdGroupSimulation",
    "AdParameter",
    "AdScheduleView",
    "AdStrengthActionItem",
    "AdvertisingPartnerIdentifier",
    "AdvertisingPartnerLinkInvitationIdentifier",
    "AgeRangeView",
    "AiMaxSearchTermAdCombinationView",
    "AndroidPrivacySharedKeyGoogleAdGroup",
    "AndroidPrivacySharedKeyGoogleCampaign",
    "AndroidPrivacySharedKeyGoogleNetworkType",
    "Asset",
    "AssetCoverage",
    "AssetFieldTypePolicySummary",
    "AssetFieldTypeView",
    "AssetGroup",
    "AssetGroupAsset",
    "AssetGroupAssetCombinationData",
    "AssetGroupListingGroupFilter",
    "AssetGroupProductGroupView",
    "AssetGroupSignal",
    "AssetGroupTopCombinationView",
    "AssetPolicySummary",
    "AssetSet",
    "AssetSetAsset",
    "AssetSetTypeView",
    "Audience",
    "BackgroundCheckVerificationArtifact",
    "BatchJob",
    "BiddingDataExclusion",
    "BiddingSeasonalityAdjustment",
    "BiddingStrategy",
    "BiddingStrategySimulation",
    "BillingSetup",
    "BusinessRegistrationCheckVerificationArtifact",
    "BusinessRegistrationDocument",
    "BusinessRegistrationNumber",
    "CallReportingSetting",
    "CallView",
    "Campaign",
    "CampaignAggregateAssetView",
    "CampaignAsset",
    "CampaignAssetSet",
    "CampaignAudienceView",
    "CampaignBidModifier",
    "CampaignBudget",
    "CampaignConversionGoal",
    "CampaignCriterion",
    "CampaignCustomizer",
    "CampaignDraft",
    "CampaignGroup",
    "CampaignLabel",
    "CampaignLifecycleGoal",
    "CampaignSearchTermInsight",
    "CampaignSearchTermView",
    "CampaignSharedSet",
    "CampaignSimulation",
    "CarrierConstant",
    "ChangeEvent",
    "ChangeStatus",
    "ChannelAggregateAssetView",
    "ClickView",
    "CombinedAudience",
    "ContactDetails",
    "ContentCriterionView",
    "ConversionAction",
    "ConversionCustomVariable",
    "ConversionGoalCampaignConfig",
    "ConversionTrackingSetting",
    "ConversionValueRule",
    "ConversionValueRuleSet",
    "CreditDetails",
    "CurrencyConstant",
    "CustomAudience",
    "CustomAudienceMember",
    "CustomConversionGoal",
    "CustomInterest",
    "CustomInterestMember",
    "CustomLeadFormSubmissionField",
    "Customer",
    "CustomerAcquisitionGoalSettings",
    "CustomerAgreementSetting",
    "CustomerAsset",
    "CustomerAssetSet",
    "CustomerClient",
    "CustomerClientLink",
    "CustomerConversionGoal",
    "CustomerCustomizer",
    "CustomerLabel",
    "CustomerLifecycleGoal",
    "CustomerManagerLink",
    "CustomerNegativeCriterion",
    "CustomerSearchTermInsight",
    "CustomerSkAdNetworkConversionValueSchema",
    "CustomerUserAccess",
    "CustomerUserAccessInvitation",
    "CustomizerAttribute",
    "DataLink",
    "DataPartnerIdentifier",
    "DetailContentSuitabilityPlacementView",
    "DetailPlacementView",
    "DetailedDemographic",
    "DisplayKeywordView",
    "DistanceView",
    "DomainCategory",
    "DynamicSearchAdsSearchTermView",
    "ExpandedLandingPageView",
    "Experiment",
    "ExperimentArm",
    "Fellowship",
    "FinalUrlExpansionAssetView",
    "GenderView",
    "GeoTargetConstant",
    "GeographicView",
    "GoogleAdsField",
    "GoogleAdsIdentifier",
    "GranularInsuranceStatus",
    "GranularLicenseStatus",
    "GroupContentSuitabilityPlacementView",
    "GroupPlacementView",
    "HotelCenterLinkInvitationIdentifier",
    "HotelGroupView",
    "HotelPerformanceView",
    "HotelReconciliation",
    "IncomeRangeView",
    "InsuranceVerificationArtifact",
    "Invoice",
    "KeywordPlan",
    "KeywordPlanAdGroup",
    "KeywordPlanAdGroupKeyword",
    "KeywordPlanCampaign",
    "KeywordPlanCampaignKeyword",
    "KeywordPlanForecastPeriod",
    "KeywordPlanGeoTarget",
    "KeywordThemeConstant",
    "KeywordView",
    "Label",
    "LandingPageView",
    "LanguageConstant",
    "LeadFormSubmissionData",
    "LeadFormSubmissionField",
    "LicenseVerificationArtifact",
    "LifeEvent",
    "ListingGroupFilterDimension",
    "ListingGroupFilterDimensionPath",
    "LocalServicesEmployee",
    "LocalServicesLead",
    "LocalServicesLeadConversation",
    "LocalServicesSettings",
    "LocalServicesVerificationArtifact",
    "LocationInterestView",
    "LocationView",
    "ManagedPlacementView",
    "MediaAudio",
    "MediaBundle",
    "MediaFile",
    "MediaImage",
    "MediaVideo",
    "MerchantCenterIdentifier",
    "MerchantCenterLinkInvitationIdentifier",
    "MessageDetails",
    "MobileAppCategoryConstant",
    "MobileDeviceConstant",
    "Note",
    "OfflineConversionAlert",
    "OfflineConversionError",
    "OfflineConversionSummary",
    "OfflineConversionUploadClientSummary",
    "OfflineConversionUploadConversionActionSummary",
    "OfflineUserDataJob",
    "OfflineUserDataJobMetadata",
    "OperatingSystemVersionConstant",
    "PaidOrganicSearchTermView",
    "ParentalStatusView",
    "PaymentsAccount",
    "PerStoreView",
    "PerformanceMaxPlacementView",
    "PhoneCallDetails",
    "ProductCategoryConstant",
    "ProductGroupView",
    "ProductLink",
    "ProductLinkInvitation",
    "QualifyingQuestion",
    "Recommendation",
    "RecommendationSubscription",
    "RemarketingAction",
    "RemarketingSetting",
    "Residency",
    "SearchTermView",
    "SharedCriterion",
    "SharedSet",
    "ShoppingPerformanceView",
    "ShoppingProduct",
    "SmartCampaignSearchTermView",
    "SmartCampaignSetting",
    "ThirdPartyAppAnalyticsLink",
    "ThirdPartyAppAnalyticsLinkIdentifier",
    "TopicConstant",
    "TopicView",
    "TravelActivityGroupView",
    "TravelActivityPerformanceView",
    "UniversityDegree",
    "UserInterest",
    "UserList",
    "UserListCustomerType",
    "UserLocationView",
    "Video",
    "VideoCustomer",
    "WebpageView",
    "YoutubeVideoIdentifier",
]
